# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import io
import bson
from contextlib import redirect_stdout
from superdesk.tests import TestCase
from superdesk import get_resource_service
from superdesk.commands.delete_archived_document import DeleteArchivedDocumentCommand


class DeleteDocTestCase(TestCase):

    def setUp(self):
        self.guid = 'urn:newsml:localhost:2016-09-12T12:11:40.160498:7237e59f-c42d-4865-aee5-e364aeb2966a'

        self.archived_only_data = [{
            "_id": bson.ObjectId("588c1b901d41c805dce70df0"),
            "guid": self.guid,
            "item_id": self.guid,
            "_current_version": 3,
            "type": "text",
            "abstract": "test",
            "state": "fetched",
            "slugline": "slugline",
            "headline": "headline",
            "flags": {"marked_archived_only": True},
            "subject": [{"qcode": "17004000", "name": "Statistics"}],
            "body_html": "Test Document body",
            'source': 'AAP',
            'linked_in_packages': [{
                "package": "urn:newsml:localhost:2017-01-28T15:28:33.535974:30c01757-cfb5-4985-a8f4-c3ecb253a244",
                "package_type": "takes"}]

        }, {
            "_id": bson.ObjectId("57d224de069b7f038e9d2a53"),
            "guid": "urn:newsml:localhost:2016-08-30T06:15:35.379754:3d68cc4c-1f16-4a7f-bfca-92ca50bcdd8f",
            "item_id": "urn:newsml:localhost:2016-08-30T06:15:35.379754:3d68cc4c-1f16-4a7f-bfca-92ca50bcdd8f",
            "_current_version": 4,
            "type": "text",
            "abstract": "test",
            "state": "fetched",
            "slugline": "slugline",
            "headline": "headline",
            "flags": {"marked_archived_only": True},
            "subject": [{"qcode": "17004000", "name": "Statistics"}],
            "body_html": "Test Document body",
            'source': 'AAP'
        }, {
            "_id": "213456",
            "guid": self.guid,
            "item_id": "urn:newsml:localhost:2017-01-28T15:28:33.535974:30c01757-cfb5-4985-a8f4-c3ecb253a244",
            "_current_version": 1,
            "type": "composite",
            "abstract": "test",
            "state": "fetched",
            "slugline": "slugline",
            "headline": "headline",
            "groups": [
                {
                    "id": "root",
                    "refs": [
                        {
                            "idRef": "main"
                        }
                    ],
                    "role": "grpRole:NEP"
                },
                {
                    "id": "main",
                    "refs": [
                        {
                            "residRef":
                                "urn:newsml:localhost:2016-09-12T12:11:40.160498:7237e59f-c42d-4865-aee5-e364aeb2966a",
                            "guid":
                                "urn:newsml:localhost:2016-09-12T12:11:40.160498:7237e59f-c42d-4865-aee5-e364aeb2966a",
                            "itemClass": "icls:text",
                            "headline": "Testing takes",
                            "type": "text",
                            "renditions": {

                            },
                            "_current_version": 3,
                            "is_published": True,
                            "slugline": "Takes test",
                            "sequence": 1,
                            "location": "archived"
                        },
                        {
                            "residRef":
                                "urn:newsml:localhost:2016-08-30T06:15:35.379754:3d68cc4c-1f16-4a7f-bfca-92ca50bcdd8f",
                            "guid":
                                "urn:newsml:localhost:2016-08-30T06:15:35.379754:3d68cc4c-1f16-4a7f-bfca-92ca50bcdd8f",
                            "itemClass": "icls:text",
                            "headline": "Testing takes",
                            "type": "text",
                            "renditions": {

                            },
                            "_current_version": 4,
                            "is_published": True,
                            "slugline": "Takes test",
                            "sequence": 2,
                            "location": "archived"
                        }
                    ],
                    "role": "grpRole:main"
                }
            ]
        }
        ]

        self.archivedService = get_resource_service('archived')

    def test_no_id_provided_exception(self):
        self.archivedService.post(self.archived_only_data)

        f = io.StringIO()
        with redirect_stdout(f):
            DeleteArchivedDocumentCommand().run([])
        s = f.getvalue()
        self.assertEqual(s, 'Please provide at least one id!\n')

    def test_wrong_id_provided_exception(self):
        self.archivedService.post(self.archived_only_data)

        f = io.StringIO()
        with redirect_stdout(f):
            DeleteArchivedDocumentCommand().run(['5880000000000'])
        s = f.getvalue()
        self.assertIn('No archived story found with given ids(s)!\n', s)

    def test_delete_non_text_document_succeeds(self):
        self.archivedService.post(self.archived_only_data)

        DeleteArchivedDocumentCommand().run(['213456'])
        cursor = self.archivedService.get(req=None, lookup={'_id': '213456'})
        self.assertEqual(0, len(cursor.docs))

    def test_delete_document_succeeds(self):
        self.archivedService.post(self.archived_only_data)
        DeleteArchivedDocumentCommand().run(['588c1b901d41c805dce70df0'])

        cursor = self.archivedService.get(req=None, lookup={'_id': '588c1b901d41c805dce70df0'})
        self.assertEqual(0, len(cursor.docs))
        cursor = self.archivedService.get(req=None, lookup={'_id': '213456'})
        self.assertEqual(0, len(cursor.docs))

    def test_delete_multiple_documents_succeeds(self):
        self.archivedService.post(self.archived_only_data)
        DeleteArchivedDocumentCommand().run(['588c1b901d41c805dce70df0', '57d224de069b7f038e9d2a53'])

        cursor = self.archivedService.get(req=None, lookup={'_id': '588c1b901d41c805dce70df0'})
        self.assertEqual(0, len(cursor.docs))
        cursor = self.archivedService.get(req=None, lookup={'_id': '57d224de069b7f038e9d2a53'})
        self.assertEqual(0, len(cursor.docs))

    def test_deleting_one_take_deletes_package_but_keeps_other_takes_succeeds(self):
        # it will delete other takes in that package
        self.archivedService.post(self.archived_only_data)
        DeleteArchivedDocumentCommand().run(['588c1b901d41c805dce70df0'])

        cursor = self.archivedService.get(req=None, lookup={'_id': '588c1b901d41c805dce70df0'})
        self.assertEqual(0, len(cursor.docs))
        cursor = self.archivedService.get(req=None, lookup={'_id': '213456'})
        self.assertEqual(0, len(cursor.docs))
        cursor = self.archivedService.get(req=None, lookup={'_id': '57d224de069b7f038e9d2a53'})
        self.assertEqual(1, len(cursor.docs))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
from uuid import uuid4

import superdesk
from werkzeug.datastructures import ImmutableMultiDict
from eve.utils import ParsedRequest
from superdesk.text_utils import get_text
from superdesk.services import BaseService
from superdesk.errors import SuperdeskApiError
from apps.auth import get_user


class ConceptItemsService(BaseService):
    """
    CRUD service for concept items
    """

    def get(self, req, lookup):
        if all([req, req.sort]):
            CASE_INSENSITIVE_COLLATION = '{"locale": "en", "strength":"1"}'
            COLLATION_SORT_ARGS = (
                'name', '-name',
                'definition_html', '-definition_html',
                'definition_text', '-definition_text'
            )

            # apply case insensitive collation only if collation was not provided explicitly
            # and sorting by `name`, `definition_html` or `definition_text` was used
            req_sort_args = [arg.strip() for arg in req.sort.split(',')]
            if [arg for arg in req_sort_args if arg in COLLATION_SORT_ARGS] and \
                    (not req.args or not req.args.get('collation')):
                if req.args:
                    req.args = req.args.to_dict()
                    req.args['collation'] = CASE_INSENSITIVE_COLLATION
                    req.args = ImmutableMultiDict(req.args)
                else:
                    req.args = ImmutableMultiDict({'collation': CASE_INSENSITIVE_COLLATION})

            # if sort by (-)definition_html was used, we'll apply sort by definition_text under the hood
            if any([arg for arg in req_sort_args if arg in ('definition_html', '-definition_html')]):
                req_sort_args = ['definition_text' if arg == 'definition_html' else arg for arg in req_sort_args]
                req_sort_args = ['-definition_text' if arg == '-definition_html' else arg for arg in req_sort_args]
                if req.args:
                    req.args = req.args.to_dict()
                    req.args['sort'] = ','.join(set(req_sort_args))
                    req.args = ImmutableMultiDict(req.args)
                else:
                    req.args = ImmutableMultiDict({'sort': ','.join(set(req_sort_args))})
                req.sort = req.args['sort']

        return super().get(req, lookup)

    def on_create(self, docs):
        for doc in docs:
            self._validate_properties(doc)
            self._validate_language(doc)
            self._setup_created_by(doc)
            self._setup_group_id(doc)
            self._fill_definition_text(doc)

    def on_replace(self, doc, original):
        self._validate_properties(doc)
        self._validate_language(doc)
        self._validate_group_id(doc)
        self._setup_created_by(doc, original)
        self._setup_updated_by(doc)
        self._fill_definition_text(doc)

    def on_update(self, updates, original):
        if 'cpnat_type' in updates:
            # we must validate `properties` even if they were not in payload
            if 'properties' not in updates and 'properties' in original:
                updates['properties'] = original['properties']
        elif 'properties' in updates:
            # we must validate `cpnat_type` even if it was not in payload
            updates['cpnat_type'] = original['cpnat_type']
        if 'cpnat_type' in updates:
            self._validate_properties(updates)

        if 'language' in updates:
            self._validate_language(updates)

        if 'definition_html' in updates:
            self._fill_definition_text(updates)

        self._setup_updated_by(updates)

    def _validate_language(self, doc):
        # fetch languages from CVs
        req = ParsedRequest()
        req.projection = json.dumps({'items.qcode': 1})

        try:
            languages = superdesk.get_resource_service('vocabularies').find_one(
                req=req, _id='languages'
            ).get('items', [])
        except AttributeError:
            raise SuperdeskApiError.badRequestError(
                message="Request is not valid",
                payload={"language": "Concept items requires 'languages' vocabulary to be set"}
            )

        languages_qcodes = [lang['qcode'] for lang in languages]

        if doc['language'] not in languages_qcodes:
            raise SuperdeskApiError.badRequestError(
                message="Request is not valid",
                payload={"language": "unallowed value '{}'".format(doc['language'])}
            )

    def _validate_group_id(self, doc):
        if 'group_id' not in doc:
            raise SuperdeskApiError.badRequestError(
                message="Request is not valid",
                payload={"group_id": "This field is required"}
            )

    def _fill_definition_text(self, doc):
        doc['definition_text'] = get_text(doc['definition_html'], content='html', lf_on_block=True).strip()

    def _validate_properties(self, doc):
        getattr(
            self,
            '_validate_{}_properties'.format(doc['cpnat_type'].rsplit(':', 1)[-1].lower())
        )(doc)

    def _validate_abstract_properties(self, doc):
        if 'properties' in doc:
            raise SuperdeskApiError.badRequestError(
                message="Request is not valid",
                payload={"properties": "field is not supported when 'cpnat_type' is 'cpnat:abstract'"}
            )

    def _validate_event_properties(self, doc):
        # Step to implement validation(s):
        # 1) define schema_event in resource (https://iptc.org/std/NewsML-G2/guidelines/#more-real-world-entities)
        # 2) create cerberus Validator based on the schema_event
        # 3) validate doc['properties'] using cerberus Validator instance
        # TODO if we want to have a case insensitive sort, please, add a proper index

        raise SuperdeskApiError.badRequestError(
            message="Request is not valid",
            payload={"cpnat_type": "concept type 'cpnat:event' is not supported"}
        )

    def _validate_geoarea_properties(self, doc):
        raise SuperdeskApiError.badRequestError(
            message="Request is not valid",
            payload={"cpnat_type": "concept type 'cpnat:geoArea' is not supported"}
        )

    def _validate_object_properties(self, doc):
        raise SuperdeskApiError.badRequestError(
            message="Request is not valid",
            payload={"cpnat_type": "concept type 'cpnat:object' is not supported"}
        )

    def _validate_organisation_properties(self, doc):
        raise SuperdeskApiError.badRequestError(
            message="Request is not valid",
            payload={"cpnat_type": "concept type 'cpnat:organisation' is not supported"}
        )

    def _validate_person_properties(self, doc):
        raise SuperdeskApiError.badRequestError(
            message="Request is not valid",
            payload={"cpnat_type": "concept type 'cpnat:person' is not supported"}
        )

    def _validate_poi_properties(self, doc):
        raise SuperdeskApiError.badRequestError(
            message="Request is not valid",
            payload={"cpnat_type": "concept type 'cpnat:poi' is not supported"}
        )

    def _setup_created_by(self, doc, original=None):
        if not original:
            doc['created_by'] = get_user().get('_id')
        else:
            # for PUT request
            doc['created_by'] = original['created_by']

    def _setup_updated_by(self, doc):
        doc['updated_by'] = get_user().get('_id')

    def _setup_group_id(self, doc):
        if 'group_id' not in doc:
            doc['group_id'] = str(uuid4())



"""Preference_Tests Class"""
# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.preferences import PreferencesService
from superdesk.tests import TestCase


class PreferenceTests(TestCase):
    """Unit Test Class related to Preference routines.

    :param TestCase: test case to use
    """

    def setUp(self):
        """Setup for test

        :param self: self
        """
        self._default_user_settings = {
            "archive:view": {
                "default": "mgrid",
                "label": "Users archive view format",
                "type": "string",
                "category": "archive",
                "allowed": [
                    "mgrid",
                    "compact"
                ],
                "view": "mgrid"
            },
            "feature:preview": {
                "category": "feature",
                "default": 'false',
                "type": "bool",
                "enabled": 'false',
                "label": "Andrew"
            },
            "email:notification": {
                "category": "notifications",
                "default": 'true',
                "type": "bool",
                "enabled": 'true',
                "label": "Send Hello"
            }
        }

        self._default_session_settings = {
            "desk:items": [],
            "stage:items": [],
            "pinned:items": [],
            "scratchpad:items": []
        }

        self._session_update = {
            "session_preferences": {
                "scratchpad:items": [123]
            }
        }

        self._user_update = {
            "user_preferences": {
                "archive:view": {
                    "label": "Testing user preferences"
                }
            }
        }

    def test_setting_partial_session_preferences_with_empty_existing(self):
        """Test case for setting partial session perferences with empty existing

        :param self: self
        """
        update = self._session_update
        existing_session_settings = {"1234": {}}

        PreferencesService.update_session_prefs(self, update, existing_session_settings, '1234')
        self.assertListEqual(update["session_preferences"]["1234"]["scratchpad:items"], [123])

    def test_setting_partial_session_preferences_with_existing(self):
        """Test case for setting partial session perferences with exiting

        :param self: self
        """
        existing_session_settings = {"1234":
                                     {
                                         "desk:items": [],
                                         "stage:items": [],
                                         "pinned:items": ['a', 'b', 'c'],
                                         "scratchpad:items": []
                                     }
                                     }

        update = self._session_update
        PreferencesService.update_session_prefs(self, update, existing_session_settings, '1234')
        self.assertListEqual(update["session_preferences"]["1234"]["scratchpad:items"], [123])

    def test_setting_partial_user_preferences_with_existing(self):
        """Test case for setting partial user preferences with existing

        :param self: self
        """
        update = self._user_update
        PreferencesService.update_user_prefs(self, update, {})
        self.assertEqual(update["user_preferences"]["archive:view"]["label"], "Testing user preferences")

    def test_setting_partial_user_preferences_with_empty_existing(self):
        """Test case for setting partial user preferences with empty existing

        :param self: self
        """
        update = self._user_update
        existing_user_settings = {
            "archive:view": {
                "default": "mgrid",
                "label": "Users archive view format",
                "type": "string",
                "category": "archive",
                "allowed": [
                    "mgrid",
                    "compact"
                ],
                "view": "mgrid"
            }
        }

        PreferencesService.update_user_prefs(self, update, existing_user_settings)
        self.assertEqual(update["user_preferences"]["archive:view"]["label"], "Testing user preferences")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from superdesk.resource import Resource

logger = logging.getLogger(__name__)


class LanguagesResource(Resource):
    schema = {
        'language': {
            'type': 'string',
            'required': True,
            'unique': True
        },
        'label': {
            'type': 'string',
            'required': True,
            'unique': True
        },
        'source': {
            'type': 'boolean',
            'default': False
        },
        'destination': {
            'type': 'boolean',
            'default': False
        }
    }

    resource_methods = ['GET']
    item_methods = []



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging

from superdesk.io.registry import register_feeding_service
from superdesk.io.feeding_services.http_base_service import HTTPFeedingServiceBase
from superdesk.errors import IngestApiError, SuperdeskIngestError
from lxml import etree

logger = logging.getLogger(__name__)
NS = {'iptc': 'http://iptc.org/std/nar/2006-10-01/'}


class APFeedingService(HTTPFeedingServiceBase):
    """
    Feeding Service class which can retrieve articles from Associated Press web service
    """

    NAME = 'ap'

    label = 'AP feed API'

    fields = HTTPFeedingServiceBase.AUTH_FIELDS + [
        {
            'id': 'idList', 'type': 'text', 'label': 'Id List',
            'placeholder': 'use coma separated ids for multiple values', 'required': False
        },
        {
            'id': 'idListType', 'type': 'choices', 'label': 'Id List Type',
            'choices': (
                ('products', 'Products'),
                ('savedsearches', 'Saved searches'),
                ('topics', 'Topics'),
                ('offerings', 'Offerings'),
                ('packages', 'Packages')
            ),
            'default': 'products',
            'required': True,
        }
    ]
    HTTP_URL = 'https://syndication.ap.org/AP.Distro.Feed/GetFeed.aspx'

    def config_test(self, provider=None):
        super().config_test(provider)

    def _update(self, provider, update):
        try:
            config = provider['config']
            id_list = config['idList']
            # before "products" was hardcoded as value for "idListType"
            id_list_type = config.get('idListType', 'products')
            if not id_list.strip():
                raise KeyError
        except KeyError:
            raise SuperdeskIngestError.notConfiguredError(Exception('idList is needed'))

        # we check if the provider has been closed since the last update
        try:
            last_closed = provider['last_closed']['closed_at']
            last_updated = provider['last_updated']
        except KeyError:
            pass
        else:
            if last_closed > last_updated and 'private' in provider:
                # we reset the private data so only last page of items will be retrieved (cf. SDESK-4372)
                logger.info("reseting private data for provider {source}".format(source=provider.get('source')))
                del provider['private']

        # we remove spaces and empty values from id_list to do a clean list
        id_list = ','.join([id_.strip() for id_ in id_list.split(',') if id_.strip()])

        params = {'idList': id_list,
                  'idListType': id_list_type,
                  'format': '5',
                  'maxItems': '25',
                  }
        try:
            min_date_time = provider['private']['min_date_time']
            sequence_number = provider['private']['sequence_number']
        except KeyError:
            # the provider is new or re-opened, we want last items
            # so we need reverse-chronological order
            chronological = False
        else:
            params['minDateTime'] = min_date_time
            params['sequenceNumber'] = sequence_number
            params['sortOrder'] = 'chronological'
            chronological = True

        r = self.get_url(params=params)

        try:
            root_elt = etree.fromstring(r.content)
        except Exception:
            raise IngestApiError.apiRequestError(Exception('error while doing the request'))

        parser = self.get_feed_parser(provider)
        items = parser.parse(root_elt, provider)
        if not chronological:
            items.reverse()

        try:
            min_date_time = root_elt.xpath('//iptc:timestamp[@role="minDateTime"]/text()', namespaces=NS)[0].strip()
            sequence_number = root_elt.xpath('//iptc:transmitId/text()', namespaces=NS)[0].strip()
        except IndexError:
            raise IngestApiError.apiRequestError(Exception('missing minDateTime or transmitId'))
        else:
            update.setdefault('private', {})
            update['private']['min_date_time'] = min_date_time
            update['private']['sequence_number'] = sequence_number

        return [items]


register_feeding_service(APFeedingService)



import os
import shutil
import tempfile
from flask import json
from unittest.mock import patch

from apps.prepopulate.app_initialize import AppInitializeWithDataCommand
from apps.prepopulate.app_scaffold_data import AppScaffoldDataCommand
from apps.prepopulate.app_initialize import fillEnvironmentVariables
from superdesk import get_resource_service, app
from superdesk.tests import TestCase


class AppInitializeWithDataCommandTestCase(TestCase):
    def _run(self, *a, **kw):
        command = AppInitializeWithDataCommand()
        return command.run(*a, **kw)

    def test_app_initialization(self):
        result = self._run()
        self.assertEqual(result, 0)

    def test_app_initialization_multiple_loads(self):
        result = self._run()
        self.assertEqual(result, 0)
        result = self._run()
        self.assertEqual(result, 0)

    def data_scaffolding_test(self):
        result = self._run(['desks', 'stages'], sample_data=True)
        self.assertEqual(result, 0)

        docs = [{
            '_id': str(x),
            'type': 'text',
            'abstract': 'test abstract {}'.format(x),
            'headline': 'test headline {}'.format(x),
            'body_html': 'test long story body {}'.format(x),
            'state': 'published'
        } for x in range(0, 40)]
        get_resource_service('published').post(docs)

        stories_per_desk = 2
        existing_desks = 18
        command = AppScaffoldDataCommand()
        result = command.run(stories_per_desk)
        self.assertEqual(result, 0)

        cursor = get_resource_service('desks').get_from_mongo(None, {})
        self.assertEqual(cursor.count(), existing_desks)

        cursor = get_resource_service('archive').get_from_mongo(None, {})
        self.assertEqual(cursor.count(), existing_desks * stories_per_desk)

    def test_sample_data(self):
        result = self._run(sample_data=True)
        self.assertEqual(result, 0)

        cursor = get_resource_service('desks').get_from_mongo(None, {})
        self.assertEqual(cursor.count(), 18)

    def test_app_initialization_index_creation(self):
        result = self._run()
        self.assertEqual(result, 0)
        result = app.data.mongo.pymongo(resource='archive').db['archive'].index_information()
        self.assertTrue('groups.refs.residRef_1' in result)
        self.assertTrue(result['groups.refs.residRef_1']['sparse'])

    @patch.dict(os.environ, {'REUTERS_USERNAME': 'r_u', 'REUTERS_PASSWORD': 'r_p'})
    def test_app_initialization_set_env_variables(self):
        item = {'username': '#ENV_REUTERS_USERNAME#', 'password': '#ENV_REUTERS_PASSWORD#'}
        crt_item = fillEnvironmentVariables(item)
        self.assertEqual(crt_item['username'], 'r_u')
        self.assertEqual(crt_item['password'], 'r_p')

    @patch.dict(os.environ, {'REUTERS_USERNAME': '', 'REUTERS_PASSWORD': 'r_p'})
    def test_app_initialization_notset_env_variables(self):
        item = {'username': '#ENV_REUTERS_USERNAME#', 'password': '#ENV_REUTERS_PASSWORD#'}
        crt_item = fillEnvironmentVariables(item)
        self.assertTrue(not crt_item)

    def test_init_keeps_user_modifications(self):
        self._run(['vocabularies'])
        urgency = self.app.data.find_one('vocabularies', req=None, _id='urgency')
        self.assertIsNotNone(urgency)
        self.assertEqual('init', urgency['_etag'])

        updates = {'display_name': 'FOO'}
        self.app.data.update('vocabularies', 'urgency', updates, urgency)

        self._run(['vocabularies'])
        urgency = self.app.data.find_one('vocabularies', req=None, _id='urgency')
        self.assertEqual('FOO', urgency['display_name'])
        self.assertNotEqual('init', urgency['_etag'])

        self._run(['vocabularies'], None, None, True)
        urgency = self.app.data.find_one('vocabularies', req=None, _id='urgency')
        self.assertEqual('Urgency', urgency['display_name'])
        self.assertEqual('init', urgency['_etag'])

    def test_init_can_combine_files_from_folders(self):
        init_dir = tempfile.mkdtemp('init', 'test')
        self.app.config.update({'INIT_DATA_PATH': init_dir})

        with open(os.path.join(init_dir, 'vocabularies.json'), 'w') as f:
            f.write(json.dumps([{'_id': 'foo'}]))
            f.flush()

        self._run(['vocabularies'])
        self.assertEqual(self.app.data.find('vocabularies', req=None, lookup={})[0].count(), 1)

        self._run(['validators'])
        self.assertGreater(self.app.data.find('validators', req=None, lookup={})[0].count(), 0)

        self.app.config.pop('INIT_DATA_PATH', None)
        shutil.rmtree(init_dir)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from ..service import ProdApiService


class PlanningService(ProdApiService):
    excluded_fields = \
        {
            '_id',
            'item_class',
            'flags',
            'lock_user',
            'lock_time',
            'lock_session',
        } | ProdApiService.excluded_fields


class EventsService(ProdApiService):
    excluded_fields = \
        {
            '_id',
            'lock_action',
            'lock_user',
            'lock_time',
            'lock_session',
        } | ProdApiService.excluded_fields


class AssignmentsService(ProdApiService):
    excluded_fields = \
        {
            'lock_action',
            'lock_user',
            'lock_time',
        } | ProdApiService.excluded_fields


class EventsHistoryService(ProdApiService):
    excluded_fields = \
        {
            'update._etag',
            'update._links',
            'update._status',
            'update._updated',
            'update._created',
            'update._id',
        } | ProdApiService.excluded_fields


class EventsFilesService(ProdApiService):
    pass



from time import time
from authlib.jose import jwt
from authlib.jose.errors import BadSignatureError, ExpiredTokenError, DecodeError
from flask import abort, make_response, jsonify
from flask import current_app as app
from eve.auth import TokenAuth

from superdesk import get_resource_privileges


class JWTAuth(TokenAuth):
    """
    Implements JWT auth logic.
    """

    def check_auth(self, token, allowed_roles, resource, method):
        """
        This function is called to check if a token is valid. Must be
        overridden with custom logic.

        :param token: token.
        :param allowed_roles: allowed user roles.
        :param resource: resource being requested.
        :param method: HTTP method being executed (POST, GET, etc.)
        """

        if not app.config.get('AUTH_SERVER_SHARED_SECRET'):
            return False

        # decode jwt
        try:
            decoded_jwt = jwt.decode(
                token,
                key=app.config.get('AUTH_SERVER_SHARED_SECRET')
            )
            decoded_jwt.validate_exp(now=time(), leeway=0)
        except (BadSignatureError, ExpiredTokenError, DecodeError):
            return False

        # authorization
        resource_privileges = get_resource_privileges(resource).get(method, None)
        if resource_privileges not in decoded_jwt.get('scope', []):
            abort(
                make_response(
                    jsonify({
                        "_status": "ERR",
                        "_error": {
                            "code": 403,
                            "message": "Invalid scope"
                        }
                    }),
                    403
                )
            )

        return True



import json
import time
import pytest
from requests.auth import _basic_auth_str
from flask import url_for

from superdesk.auth_server.scopes import Scope

from ..conftest import get_test_prodapi_app


RESOURCES = (
    'archive',
    'assignments',
    'contacts',
    'desks',
    'events',
    'planning',
    'users'
)

SCOPES = tuple(i.name for i in Scope)


def test_not_authenticated(prodapi_app, prodapi_client):
    """
    Send a not authenticated request:
        - make a request without a token
        - ensure that request is not authenticated by prod api

    :param prodapi_app: prod api app
    :type prodapi_app: eve.flaskapp.Eve
    :param prodapi_client: client for prod api
    :type prodapi_client: flask.testing.FlaskClient
    """

    with prodapi_app.test_request_context():
        for resource in RESOURCES:
            # we send a request without a token
            resp = prodapi_client.get(
                url_for('{}|resource'.format(resource))
            )
            resp_data = json.loads(resp.data.decode('utf-8'))
            # we get a 401 response
            assert resp.status_code == 401
            assert resp_data['_status'] == 'ERR'


@pytest.mark.parametrize('auth_server_registered_clients', [(('ARCHIVE_READ',),)], indirect=True)
def test_authenticated(superdesk_app, superdesk_client, auth_server_registered_clients):
    """
    Send an authenticated request:
        - register client for auth server
        - retrieve an acceess token from auth server by providing `id` and `password`
        - make a request with a token
        - ensure that request is authenticated by prod api

    :param superdesk_app: superdesk api app
    :type superdesk_app: eve.flaskapp.Eve
    :param superdesk_client: client for superdesk api
    :type superdesk_client: flask.testing.FlaskClient
    """

    client_id = auth_server_registered_clients[0]['client_id']
    password = auth_server_registered_clients[0]['password']

    # we send a client id and password to get an access token
    with superdesk_app.test_request_context():
        resp = superdesk_client.post(
            url_for('auth_server.issue_token'),
            data={
                'grant_type': 'client_credentials'
            },
            headers={
                'Authorization': _basic_auth_str(client_id, password)
            }
        )

    # we get an access token
    resp_data = json.loads(resp.data.decode('utf-8'))
    assert resp.status_code == 200
    assert resp_data['token_type'] == 'Bearer'
    assert 'access_token' in resp_data

    # we create a prodapi flask app and client
    prodapi_app = get_test_prodapi_app()
    prodapi_client = prodapi_app.test_client()

    # we send a request with an auth token
    token = resp_data['access_token']
    with prodapi_app.test_request_context():
        for resource in ('archive',):
            # we send a request with a token
            resp = prodapi_client.get(
                url_for('{}|resource'.format(resource)),
                headers={
                    'Authorization': 'Bearer {}'.format(token)
                }
            )

            # we get a 200 response
            assert resp.status_code == 200


@pytest.mark.parametrize('auth_server_registered_clients', [(('ARCHIVE_READ',),)], indirect=True)
def test_bad_shared_key(superdesk_app, superdesk_client, auth_server_registered_clients):
    """
    Send a request with a token which is signed and verified using different secrets:
        - register client for auth server
        - retrieve an acceess token from auth server by providing `id` and `password`
        - make a request with a token
        - ensure that request is NOT authenticated by prod api, because of different shared secret

    :param superdesk_app: superdesk api app
    :type superdesk_app: eve.flaskapp.Eve
    :param superdesk_client: client for superdesk api
    :type superdesk_client: flask.testing.FlaskClient
    """

    client_id = auth_server_registered_clients[0]['client_id']
    password = auth_server_registered_clients[0]['password']

    # we send a client id and password to get an access token
    with superdesk_app.test_request_context():
        resp = superdesk_client.post(
            url_for('auth_server.issue_token'),
            data={
                'grant_type': 'client_credentials'
            },
            headers={
                'Authorization': _basic_auth_str(client_id, password)
            }
        )

    # we get an access token
    resp_data = json.loads(resp.data.decode('utf-8'))
    token = resp_data['access_token']

    # we create a prodapi flask app and client
    prodapi_app = get_test_prodapi_app()
    prodapi_client = prodapi_app.test_client()

    # we change `AUTH_SERVER_SHARED_SECRET`
    prodapi_app.config['AUTH_SERVER_SHARED_SECRET'] = '7fZOf0VI9T70vU5uNlKLrc5GlabxVgl6'

    # we send a request with an auth token
    with prodapi_app.test_request_context():
        for resource in ('archive',):
            # we send a request with a token
            resp = prodapi_client.get(
                url_for('{}|resource'.format(resource)),
                headers={
                    'Authorization': 'Bearer {}'.format(token)
                }
            )
            resp_data = json.loads(resp.data.decode('utf-8'))

    # we get a 401 response
    assert resp.status_code == 401
    assert resp_data == {
        '_status': 'ERR',
        '_error': {
            'code': 401,
            'message': 'Please provide proper credentials'
        }
    }


@pytest.mark.parametrize('issued_tokens', [(SCOPES, tuple())], indirect=True)
def test_scopes(issued_tokens, prodapi_app, prodapi_client):
    """
    Send a request with 'all scopes' token:
        - make a requests to all endpoints with a token
        - ensure that all requests are authorized by prod api

    Send a request with 'empty scopes' token:
        - make a requests to all endpoints with a token
        - ensure that all requests are NOT authorized by prod api due to empty scopes

    :param issued_tokens: list of tokens provided by `issued_tokens` fixture
    :type issued_tokens: list
    :param prodapi_app: prod api app
    :type prodapi_app: eve.flaskapp.Eve
    :param prodapi_client: client for prod api
    :type prodapi_client: flask.testing.FlaskClient
    """

    access_token_all_scopes = issued_tokens[0]['access_token']
    access_token_no_scopes = issued_tokens[1]['access_token']

    with prodapi_app.test_request_context():
        for resource in RESOURCES:
            # we send a requests with 'empty scopes' token
            resp = prodapi_client.get(
                url_for('{}|resource'.format(resource)),
                headers={
                    'Authorization': 'Bearer {}'.format(access_token_no_scopes)
                }
            )
            resp_data = json.loads(resp.data.decode('utf-8'))
            # we get a 403 response
            assert resp.status_code == 403
            assert resp_data == {
                '_status': 'ERR',
                '_error': {
                    'code': 403,
                    'message': 'Invalid scope'
                }
            }
            # we send a requests with 'all scopes' token
            resp = prodapi_client.get(
                url_for('{}|resource'.format(resource)),
                headers={
                    'Authorization': 'Bearer {}'.format(access_token_all_scopes)
                }
            )
            # we get a 200 response
            assert resp.status_code == 200


@pytest.mark.parametrize('issued_tokens', [(("DESKS_READ",),)], indirect=True)
def test_scopes_partial(issued_tokens, prodapi_app):
    """
    Send a request with DESKS_READ scope token to `desks` and `items`:
        - make requests to `desks` and `items` endpoints
        - ensure that only request to `desk` is authorized

    :param prodapi_app: prod api app
    :type prodapi_app: eve.flaskapp.Eve
    :param issued_tokens: list of tokens provided by `issued_tokens` fixture
    :type issued_tokens: list
    """

    access_token = issued_tokens[0]['access_token']
    prodapi_client = prodapi_app.test_client()

    with prodapi_app.test_request_context():
        # we send a request with 'DESKS_READ scope' token to desks endpoint
        resp = prodapi_client.get(
            url_for('desks|resource'),
            headers={
                'Authorization': 'Bearer {}'.format(access_token)
            }
        )
        # we get a 200 response
        assert resp.status_code == 200

        # we send a request with 'DESKS_READ scope' token to items endpoint
        resp = prodapi_client.get(
            url_for('archive|resource'),
            headers={
                'Authorization': 'Bearer {}'.format(access_token)
            }
        )
        resp_data = json.loads(resp.data.decode('utf-8'))
        # we get a 403 response
        assert resp.status_code == 403
        assert resp_data == {
            '_status': 'ERR',
            '_error': {
                'code': 403,
                'message': 'Invalid scope'
            }
        }


@pytest.mark.parametrize('auth_server_registered_clients', [(('ARCHIVE_READ',),)], indirect=True)
@pytest.mark.parametrize('superdesk_app', [{'AUTH_SERVER_EXPIRATION_DELAY': 1}, ], indirect=True)
def test_token_expired(superdesk_app, superdesk_client, auth_server_registered_clients):
    """
    Send a request with an expired token:
        - register client for auth server
        - retrieve an acceess token from auth server by providing `id` and `password`
        - wait until token expires
        - make a request with an expired token
        - ensure that request is NOT authenticated by prod api

    :param superdesk_app: superdesk api app
    :type superdesk_app: eve.flaskapp.Eve
    :param superdesk_client: client for superdesk api
    :type superdesk_client: flask.testing.FlaskClient
    """

    client_id = auth_server_registered_clients[0]['client_id']
    password = auth_server_registered_clients[0]['password']
    expiration_delay = superdesk_app.config['AUTH_SERVER_EXPIRATION_DELAY']

    # we send a client id and password to get an access token
    with superdesk_app.test_request_context():
        resp = superdesk_client.post(
            url_for('auth_server.issue_token'),
            data={
                'grant_type': 'client_credentials'
            },
            headers={
                'Authorization': _basic_auth_str(client_id, password)
            }
        )

    # we get an access token
    assert resp.status_code == 200
    resp_data = json.loads(resp.data.decode('utf-8'))
    token = resp_data['access_token']

    # wait until token expire
    time.sleep(expiration_delay + 1)

    # we create a prodapi flask app and client
    prodapi_app = get_test_prodapi_app()
    prodapi_client = prodapi_app.test_client()

    # we send a request with an expired auth token
    with prodapi_app.test_request_context():
        for resource in ('archive',):
            # we send a request with a token
            resp = prodapi_client.get(
                url_for('{}|resource'.format(resource)),
                headers={
                    'Authorization': 'Bearer {}'.format(token)
                }
            )

    # we get a 401 response
    assert resp.status_code == 401



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
import copy
from io import BytesIO
from unittest.mock import MagicMock

import magic
import requests_mock
from werkzeug.datastructures import FileStorage

import superdesk
from superdesk import config
from superdesk.errors import SuperdeskApiError
from superdesk.tests import TestCase


class Req(dict):
    args = {}


video_info = {
    '_id': 'video_id',
}


class VideoEditTestCase(TestCase):
    project_data = {
        '_id': 'video_id',
        'metadata': {
            "codec_name": "h264",
            "width": 1280,
            "height": 720,
            "duration": 10,
        },
        'url': 'video_url',
        'mime_type': 'video/mp4',
        'version': 1,
    }

    def setUp(self):
        self.video_edit = superdesk.get_resource_service('video_edit')
        self.app.config['VIDEO_SERVER_ENABLED'] = 'true'
        self.app.config['VIDEO_SERVER_URL'] = 'http://localhost'
        with requests_mock.mock() as mock:
            dirname = os.path.dirname(os.path.realpath(__file__))
            video_file_path = os.path.normpath(os.path.join(dirname, 'fixtures', 'envy_dog.mp4'))

            with open(video_file_path, 'rb') as f:
                doc = {'media': FileStorage(BytesIO(f.read()), 'video.mp4', content_type='video/mp4')}

            mock.post('http://localhost/projects/', json=self.project_data)
            mock.get("http://localhost/projects/video_id/thumbnails?type=timeline&amount=60", json={"processing": True})
            archive_service = superdesk.get_resource_service('archive')
            magic.from_buffer = MagicMock()
            magic.from_buffer.return_value = 'video/mp4'
            self.item = archive_service.find_one(req=None, _id=archive_service.post([doc])[0])

    def test_get_video(self):
        with requests_mock.mock() as mock:
            mock.get('http://localhost/projects/video_id', json=video_info)
            res = self.video_edit.find_one(Req(), _id=self.item['_id'])
            self.assertEqual(res['project']['_id'], video_info['_id'])

    def test_upload_video(self):
        self.assertEqual(self.item["renditions"], {'original': {'href': 'video_url',
                                                                'mimetype': 'video/mp4',
                                                                'version': 1,
                                                                'video_editor_id': 'video_id'}})

    def test_missing_video_id(self):
        doc = {
            'item': {
                config.ID_FIELD: '123',
                'renditions': {
                    'original': {}
                }
            }
        }
        with self.assertRaises(SuperdeskApiError) as ex:
            self.video_edit.create([doc])
        self.assertEqual(ex.exception.message, '"video_editor_id" is required')

    def test_missing_action(self):
        doc = {
            'item': {
                config.ID_FIELD: '123',
                'renditions': {
                    'original': {'video_editor_id': 'video_id'}
                }
            }
        }
        with self.assertRaises(SuperdeskApiError) as ex:
            self.video_edit.create([doc])
        self.assertEqual(ex.exception.message, '"capture" or "edit" is required')

    def test_edit_video(self):
        project_data = copy.deepcopy(self.project_data)
        doc = {
            "item": {
                config.ID_FIELD: self.item[config.ID_FIELD],
                'renditions': self.item['renditions'],
            },
            "edit": {
                "crop": "0,0,200,500",
                "rotate": -90,
                "trim": "5,15"
            },
        }
        with requests_mock.mock() as mock:
            project_data['version'] = 2
            mock.get('http://localhost/projects/video_id', json=project_data)
            mock.post('http://localhost/projects/video_id/duplicate', json=project_data)
            mock.put('http://localhost/projects/video_id', json={"processing": True})
            item = self.video_edit.find_one(req=None, _id=self.video_edit.create([doc])[0])
            self.assertEqual(
                item["renditions"],
                {
                    'original': {
                        'href': 'video_url',
                        'mimetype': 'video/mp4',
                        'version': 3,
                        'video_editor_id': 'video_id'
                    }
                }
            )

    def test_capture_thumbnail(self):
        doc = {
            "item": {
                config.ID_FIELD: self.item[config.ID_FIELD],
                'renditions': self.item['renditions'],
            },
            "capture": {
                "crop": "0,0,200,500",
                "rotate": -90,
                "trim": "5,10"
            },
        }
        project_data = copy.deepcopy(self.project_data)
        project_data.setdefault('thumbnails', {})['preview'] = {
            "mimetype": "image/png",
            "url": "http://localhost/projects/video_id/raw/thumbnails/preview"
        }
        with requests_mock.mock() as mock:
            mock.get('http://localhost/projects/video_id', json=project_data)
            mock.post('http://localhost/projects/video_id', json={"processing": True})
            mock.get('http://localhost/projects/video_id/thumbnails?type=preview&crop=0,0,200,500&rotate=-90',
                     json=project_data)
            item = self.video_edit.find_one(req=None, _id=self.video_edit.create([doc])[0])
            self.assertEqual(item["renditions"],
                             {'original': {'href': 'video_url',
                                           'mimetype': 'video/mp4',
                                           'version': 1,
                                           'video_editor_id':
                                           'video_id'},
                              'thumbnail': {
                                 "mimetype": "image/png",
                                 "href": "http://localhost/projects/video_id/raw/thumbnails/preview"
                             }})

    def test_upload_thumbnail(self):
        thumbnail = {
            "mimetype": "image/jpeg",
            "url": "http://localhost/projects/video_id/raw/thumbnails/preview"
        }
        with requests_mock.mock() as mock:
            mock.post('http://localhost/projects/video_id/thumbnails', json=thumbnail)
            req = {'file': FileStorage(BytesIO(b'abcdef'), 'image.jpeg')}
            res = self.video_edit.on_replace(req, {'project': {'_id': 'video_id'}})
            self.assertEqual(
                res['renditions']['thumbnail']['href'],
                'http://localhost/projects/video_id/raw/thumbnails/preview'
            )
            self.assertEqual(res['renditions']['thumbnail']['mimetype'], 'image/jpeg')

    def test_capture_timeline(self):
        with requests_mock.mock() as mock:
            mock.get(
                'http://localhost/projects/video_id/thumbnails?type=timeline&amount=60',
                json={'processing': True},
                status_code=202,
            )
            req = Req()
            setattr(req, 'args', {'action': 'timeline'})
            res = self.video_edit.find_one(req, _id=self.item['_id'])
            self.assertEqual(res[config.ID_FIELD], video_info['_id'])
            self.assertTrue(res['processing'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from datetime import timedelta

from apps.archive.common import is_genre, BROADCAST_GENRE, get_utc_schedule
from superdesk.utc import utcnow


class ArchiveCommonTestCase(unittest.TestCase):

    def test_broadcast_content(self):
        content = {
            'genre': [{'name': 'Broadcast Script', 'qcode': 'Broadcast Script'}]
        }

        self.assertTrue(is_genre(content, BROADCAST_GENRE))

    def test_broadcast_content_if_genre_is_none(self):
        content = {
            'genre': None
        }

        self.assertFalse(is_genre(content, BROADCAST_GENRE))

    def test_broadcast_content_if_genre_is_empty_list(self):
        content = {
            'genre': []
        }

        self.assertFalse(is_genre(content, BROADCAST_GENRE))

    def test_broadcast_content_if_genre_is_other_than_broadcast(self):
        content = {
            'genre': [{'name': 'Article', 'qcode': 'Article'}]
        }

        self.assertFalse(is_genre(content, BROADCAST_GENRE))
        self.assertTrue(is_genre(content, 'Article'))

    def test_get_utc_schedule(self):
        embargo_date = utcnow() + timedelta(minutes=10)
        content = {
            'embargo': embargo_date
        }
        utc_schedule = get_utc_schedule(content, 'embargo')
        self.assertEqual(utc_schedule, embargo_date)




import unittest
from datetime import datetime

from superdesk.io.subjectcodes import SubjectIndex
from superdesk.io.iptc import init_app


class SubjectcodeTestCase(unittest.TestCase):

    def setUp(self):
        self.subjects = SubjectIndex()

    def test_subjectcodes_empty(self):
        self.assertEqual([], self.subjects.get_items())

    def test_subjectcodes_register(self):
        self.subjects.register({
            '01000000': 'arts',
        }, last_modified=datetime(2015, 10, 10))

        self.assertEqual(1, len(self.subjects.get_items()))

        self.subjects.register({
            '01001000': 'archeology',
        }, last_modified=datetime(2015, 1, 1))

        self.assertEqual(2, len(self.subjects.get_items()))
        self.assertEqual(datetime(2015, 10, 10), self.subjects.last_modified)
        self.assertEqual('01000000', self.subjects.get_items()[1]['parent'])

        self.subjects.register({
            'subj:test': 'test'
        })

        self.assertEqual(3, len(self.subjects.get_items()))
        self.assertEqual(None, self.subjects.get_items()[2]['parent'])

    def test_iptc_init(self):
        init_app(self)
        self.assertEqual(1404, len(self.subjects.get_items()))



import superdesk
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, CONTENT_STATE, ITEM_STATE, get_schema
from flask import render_template, render_template_string
from superdesk.errors import SuperdeskApiError
from eve.utils import config
from flask import current_app as app
from .service import get_highlight_name
from apps.archive.common import ITEM_EXPORT_HIGHLIGHT, ITEM_CREATE_HIGHLIGHT

PACKAGE_FIELDS = {
    'type', 'state', 'groups', 'unique_name', 'pubstatus', 'origina_creator', 'flags', 'guid',
    'schedule_settings', 'expiry', 'format', 'lock_time', 'lock_user', 'lock_session', config.ID_FIELD,
    config.LAST_UPDATED, config.DATE_CREATED, config.ETAG, 'version', '_current_version', 'version_creator',
    'operation', 'unique_id', 'version_created', 'fields_meta',
}


def get_template(highlightId):
    """Return the string template associated with highlightId or none"""
    if not highlightId:
        return None
    highlightService = superdesk.get_resource_service('highlights')
    highlight = highlightService.find_one(req=None, _id=highlightId)
    if not highlight or not highlight.get('template'):
        return None

    templateService = superdesk.get_resource_service('content_templates')
    template = templateService.find_one(req=None, _id=highlight.get('template'))
    return template


class GenerateHighlightsService(superdesk.Service):
    def create(self, docs, **kwargs):
        """Generate highlights text item for given package.

        If doc.preview is True it won't save the item, only return.
        """
        service = superdesk.get_resource_service('archive')
        for doc in docs:
            preview = doc.get('preview', False)
            package = service.find_one(req=None, _id=doc['package'])
            if not package:
                superdesk.abort(404)
            export = doc.get('export')
            template = get_template(package.get('highlight'))
            stringTemplate = None
            if template and template.get('data') and template['data'].get('body_html'):
                stringTemplate = template['data']['body_html']

            doc.clear()
            doc[ITEM_TYPE] = CONTENT_TYPE.TEXT
            doc['family_id'] = package.get('guid')
            doc[ITEM_STATE] = CONTENT_STATE.SUBMITTED
            doc[config.VERSION] = 1

            for field in package:
                if field not in PACKAGE_FIELDS:
                    doc[field] = package[field]

            items = []
            for group in package.get('groups', []):
                for ref in group.get('refs', []):
                    if 'residRef' in ref:
                        item = service.find_one(req=None, _id=ref.get('residRef'))
                        if item:
                            if not (export or preview) and \
                                    (item.get('lock_session') or item.get('state') != 'published'):
                                message = 'Locked or not published items in highlight list.'
                                raise SuperdeskApiError.forbiddenError(message)

                            items.append(item)
                            if not preview:
                                app.on_archive_item_updated(
                                    {'highlight_id': package.get('highlight'),
                                     'highlight_name': get_highlight_name(package.get('highlight'))}, item,
                                    ITEM_EXPORT_HIGHLIGHT)

            if stringTemplate:
                doc['body_html'] = render_template_string(stringTemplate, package=package, items=items)
            else:
                doc['body_html'] = render_template('default_highlight_template.txt', package=package, items=items)
        if preview:
            return ['' for doc in docs]
        else:
            ids = service.post(docs, **kwargs)
            for id in ids:
                app.on_archive_item_updated(
                    {'highlight_id': package.get('highlight'),
                     'highlight_name': get_highlight_name(package.get('highlight'))}, {'_id': id},
                    ITEM_CREATE_HIGHLIGHT)
            return ids


class GenerateHighlightsResource(superdesk.Resource):
    """Generate highlights item for given package."""

    schema = {
        'package': {
            # not setting relation here, we will fetch it anyhow
            'type': 'string',
            'required': True,
        },
        'preview': {
            'type': 'boolean',
            'default': False,
        },
        'export': {
            'type': 'boolean',
            'default': False,
        },
        'type': {
            'type': 'string',
            'readonly': True,
        },
    }
    schema.update(get_schema(versioning=True))

    resource_methods = ['POST']
    item_methods = []
    privileges = {'POST': 'highlights'}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from superdesk import config
from superdesk.tests import TestCase

from superdesk.etree import etree
from superdesk.io.feed_parsers.nitf import NITFFeedParser


class NITFTestCase(TestCase):

    vocab = [{'_id': 'genre', 'items': [{'name': 'Current'}]}]

    def setUp(self):
        with self.app.app_context():
            self.app.data.insert('vocabularies', self.vocab)
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
        provider = {'name': 'Test'}
        with open(fixture, 'rb') as f:
            self.nitf = f.read()
            self.item = NITFFeedParser().parse(etree.fromstring(self.nitf), provider)


class AAPTestCase(NITFTestCase):

    filename = 'aap.xml'

    def test_headline(self):
        self.assertEqual(self.item.get('headline'), "The main stories on today's 1900 ABC TV news")

    def test_slugline(self):
        self.assertEqual(self.item.get('slugline'), 'Monitor 1900 ABC News')

    def test_subjects(self):
        self.assertEqual(len(self.item.get('subject')), 4)
        self.assertIn({'qcode': '02000000', 'name': 'Justice'}, self.item.get('subject'))
        self.assertIn({'qcode': '02003000', 'name': 'Police'}, self.item.get('subject'))
        self.assertIn({'qcode': '02003001', 'name': 'law enforcement'}, self.item.get('subject'))
        self.assertIn({'qcode': '02003002', 'name': 'investigation'}, self.item.get('subject'))

    def test_guid(self):
        self.assertEqual(self.item.get('guid'), 'AAP.115314987.5417374')
        self.assertEqual(self.item.get('guid'), self.item.get('uri'))

    def test_type(self):
        self.assertEqual(self.item.get('type'), 'text')

    def test_urgency(self):
        self.assertEqual(self.item.get('urgency'), 5)

    def test_dateline(self):
        self.assertEqual(self.item.get('dateline', {}).get('located', {}).get('city'), 'Sydney')

    def test_byline(self):
        self.assertEqual(self.item.get('byline'), 'By John Doe')

    def test_abstract(self):
        self.assertEqual(self.item.get('abstract'), 'The main stories on today\'s 1900 ABC TV news')

    def test_dates(self):
        self.assertEqual(self.item.get('firstcreated').isoformat(), '2013-10-20T08:27:51+00:00')
        self.assertEqual(self.item.get('versioncreated').isoformat(), '2013-10-20T08:27:51+00:00')

    def test_content(self):
        text = "<p>   1A) More extreme weather forecast over the next few days the <br/>fire situation is likely"
        self.assertIn(text, self.item.get('body_html'))
        self.assertIsInstance(self.item.get('body_html'), type(''))
        self.assertNotIn('<body.content>', self.item.get('body_html'))

    def test_pubstatus(self):
        self.assertEqual('usable', self.item.get('pubstatus'))

    def test_ingest_provider_sequence(self):
        self.assertEqual(self.item.get('ingest_provider_sequence'), '1747')

    def test_anpa_category(self):
        self.assertEqual(self.item.get('anpa_category')[0]['qcode'], 'a')

    def test_word_count(self):
        self.assertIsInstance(self.item.get('word_count'), int)


class APExampleTestCase(NITFTestCase):

    filename = 'ap-nitf.xml'

    def test_headline(self):
        self.assertEqual(self.item.get('headline'), 'Can trading pollution like stocks help fight climate change?')

    def test_byline(self):
        self.assertEqual(self.item.get('byline'), 'By BERNARD CONDON')

    def test_ednote(self):
        self.assertEqual(self.item.get('ednote'), 'For global distribution')

    def test_word_count(self):
        self.assertIsInstance(self.item.get('word_count'), int)


class IPTCExampleTestCase(NITFTestCase):

    filename = 'nitf-fishing.xml'

    def test_headline(self):
        self.assertEqual(self.item.get('headline'), 'Weather and Tide Updates for Norfolk')

    def test_pubstatus(self):
        self.assertEqual('canceled', self.item.get('pubstatus'))

    def test_guid(self):
        self.assertEqual('iptc.321656141.b', self.item.get('guid'))

    def test_subjects(self):
        self.assertEqual(3, len(self.item.get('subject')))

    def test_place(self):
        places = self.item.get('place', [])
        self.assertEqual(1, len(places))
        self.assertEqual('Norfolk', places[0]['name'])
        self.assertEqual('US', places[0]['code'])

    def test_expiry(self):
        self.assertEqual('2012-02-26T14:30:00+00:00', self.item.get('expiry').isoformat())

    def test_keywords(self):
        self.assertIn('fishing', self.item.get('keywords'))

    def test_ednote(self):
        self.assertEqual('Today begins an expanded format of this popular column.', self.item.get('ednote'))

    def test_byline(self):
        self.assertEqual('By Alan Karben', self.item.get('byline'))

    def test_word_count(self):
        self.assertIsInstance(self.item.get('word_count'), int)


class PATestCase(NITFTestCase):

    filename = 'pa1.xml'

    def test_headline(self):
        self.assertEqual(self.item.get('headline'), 'PA SPORT TRIVIA (OCTOBER 14)')

    def test_pubstatus(self):
        self.assertEqual('usable', self.item.get('pubstatus'))

    def test_guid(self):
        self.assertEqual('af1f7ad5-5619-49de-84cc-2e608538c77fSSS-3-1', self.item.get('guid'))
        self.assertEqual(self.item.get('format'), 'HTML')

    def test_subjects(self):
        self.assertEqual(4, len(self.item.get('subject')))

    def test_keywords(self):
        self.assertIn('Trivia (Oct 14)', self.item.get('keywords'))

    def test_word_count(self):
        self.assertIsInstance(self.item.get('word_count'), int)


class PATestCase2(NITFTestCase):

    filename = 'pa2.xml'

    def test_headline(self):
        self.assertEqual(self.item.get('headline'), '1 SOCCER INT-Teams')

    def test_pubstatus(self):
        self.assertEqual('usable', self.item.get('pubstatus'))

    def test_guid(self):
        self.assertEqual('T201510140143580001T', self.item.get('guid'))
        self.assertEqual(self.item.get('format'), 'preserved')

    def test_guid_2(self):
        self.assertEqual(self.item.get('type'), 'text')

    def test_subjects(self):
        self.assertEqual(0, len(self.item.get('subject')))

    def test_keywords(self):
        self.assertIn('INT-Teams', self.item.get('keywords'))

    def test_word_count(self):
        self.assertEqual(58, self.item.get('word_count'))


class ParseSubjects(TestCase):
    def test_get_subjects(self):
        xml = (b'<?xml version="1.0" encoding="UTF-8"?>'
               b'<nitf><head>'
               b'<tobject tobject.type="News">'
               b'<tobject.property tobject.property.type="Current" />'
               b'<tobject.subject tobject.subject.refnum="02003000" '
               b'tobject.subject.type="Justice" tobject.subject.matter="Police" />'
               b'</tobject></head></nitf>')
        subjects = NITFFeedParser().get_subjects(etree.fromstring(xml))
        self.assertEqual(len(subjects), 2)
        self.assertIn({'qcode': '02000000', 'name': 'Justice'}, subjects)
        self.assertIn({'qcode': '02003000', 'name': 'Police'}, subjects)

    def test_get_subjects_with_invalid_qcode(self):
        xml = (b'<?xml version="1.0" encoding="UTF-8"?>'
               b'<nitf><head>'
               b'<tobject tobject.type="News">'
               b'<tobject.property tobject.property.type="Current" />'
               b'<tobject.subject tobject.subject.refnum="00000000" '
               b'tobject.subject.type="Justice" tobject.subject.matter="Police" />'
               b'</tobject></head></nitf>')
        subjects = NITFFeedParser().get_subjects(etree.fromstring(xml))
        self.assertEqual(len(subjects), 0)


class MappingTestCase(TestCase):

    filename = 'mapping_test.xml'
    mapping = {
        'subject': {
            'update': True,
            'key_hook': lambda item, value: item.setdefault('subject', []).extend(value)
        },
        'subject_test': {
            'callback': lambda _: ['TEST OK'],
            'key_hook': lambda item, value: item.setdefault('subject', []).extend(value)
        },
    }

    def setUp(self):
        config.NITF_MAPPING = self.mapping
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
        provider = {'name': 'Test'}
        with open(fixture, 'rb') as f:
            self.nitf = f.read()
            self.item = NITFFeedParser().parse(etree.fromstring(self.nitf), provider)

    def test_update_and_hook(self):
        subjects = self.item.get('subject')
        # have we got both items ?
        self.assertEqual(len(subjects), 2)
        # the initial updated subject need to be here
        self.assertIn({'qcode': '02000000', 'name': 'Kriminalitet og rettsvesen'}, subjects)
        # and our key from subject_test need to be here too
        self.assertIn('TEST OK', subjects)

    def tearDown(self):
        del config.NITF_MAPPING


class HandleInvalidFieldsTestCase(NITFTestCase):

    filename = 'handle_invalid_fields.xml'

    def test_dates(self):
        self.assertEqual(self.item.get('firstcreated').isoformat(), '2020-07-03T20:51:36+00:00')
        self.assertEqual(self.item.get('versioncreated').isoformat(), '2020-07-03T20:51:36+00:00')

    def test_expiry(self):
        self.assertEqual(self.item.get('expiry'), None)

    def test_keywords(self):
        self.assertEqual(len(self.item.get('keywords')), 0)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from collections import namedtuple

from superdesk.resource import Resource, not_analyzed, not_indexed, not_enabled
from .packages import LINKED_IN_PACKAGES, PACKAGE
from eve.utils import config
from superdesk.utils import SuperdeskBaseEnum

GUID_TAG = 'tag'
GUID_FIELD = 'guid'
GUID_NEWSML = 'newsml'
INGEST_ID = 'ingest_id'
INGEST_VERSION = 'ingest_version'
FAMILY_ID = 'family_id'
ASSOCIATIONS = 'associations'


#: item public states
pub_status = ['usable', 'withheld', 'canceled']
PUB_STATUS = namedtuple('PUBSTATUS', ['USABLE', 'HOLD', 'CANCELED'])(*pub_status)

ITEM_TYPE = 'type'
content_type = ['text', 'preformatted', 'audio', 'video', 'picture', 'graphic', 'composite', 'event']
CONTENT_TYPE = namedtuple('CONTENT_TYPE',
                          ['TEXT', 'PREFORMATTED', 'AUDIO', 'VIDEO',
                           'PICTURE', 'GRAPHIC', 'COMPOSITE', 'EVENT'])(*content_type)

MEDIA_TYPES = ('audio', 'video', 'picture', 'graphic')

ITEM_STATE = 'state'
ITEM_PRIORITY = 'priority'
ITEM_URGENCY = 'urgency'

#: item internal states
content_state = ['draft', 'ingested', 'routed', 'fetched', 'submitted', 'in_progress', 'spiked',
                 'published', 'killed', 'corrected', 'scheduled', 'recalled', 'unpublished']
CONTENT_STATE = namedtuple('CONTENT_STATE', ['DRAFT', 'INGESTED', 'ROUTED', 'FETCHED', 'SUBMITTED', 'PROGRESS',
                                             'SPIKED', 'PUBLISHED', 'KILLED', 'CORRECTED',
                                             'SCHEDULED', 'RECALLED', 'UNPUBLISHED'])(*content_state)
PUBLISH_STATES = {
    CONTENT_STATE.PUBLISHED,
    CONTENT_STATE.SCHEDULED,
    CONTENT_STATE.CORRECTED,
    CONTENT_STATE.KILLED,
    CONTENT_STATE.RECALLED,
    CONTENT_STATE.UNPUBLISHED,
}

FORMAT = 'format'
formats = ['HTML', 'preserved']
FORMATS = namedtuple('FORMAT', ['HTML', 'PRESERVED'])(*formats)

BYLINE = 'byline'
SIGN_OFF = 'sign_off'
EMBARGO = 'embargo'
PUBLISH_SCHEDULE = 'publish_schedule'
SCHEDULE_SETTINGS = 'schedule_settings'
PROCESSED_FROM = 'processed_from'

# part the task dict
LAST_DESK = 'last_desk'
LAST_AUTHORING_DESK = 'last_authoring_desk'
LAST_PRODUCTION_DESK = 'last_production_desk'
DESK_HISTORY = 'desk_history'

ITEM_EVENT_ID = 'event_id'

geopoint = {
    'type': 'dict',
    'mapping': {'type': 'geo_point'},
    'nullable': True,
    'schema': {
        'lat': {'type': 'float'},
        'lon': {'type': 'float'},
    },
}

metadata_schema = {
    config.ID_FIELD: {
        'type': 'string',
        'unique': True
    },
    #: Identifiers
    'guid': {
        'type': 'string',
        'unique': True,
        'mapping': not_analyzed
    },
    'uri': {
        'type': 'string',
        'mapping': not_analyzed,
    },
    'unique_id': {
        'type': 'integer',
        'unique': True,
    },
    'unique_name': {
        'type': 'string',
        'unique': True,
        'mapping': not_analyzed
    },
    'version': {
        'type': 'integer'
    },
    'ingest_id': {
        'type': 'string',
        'mapping': not_analyzed
    },
    'ingest_version': {
        'type': 'string',
        'mapping': not_analyzed
    },
    'family_id': {
        'type': 'string',
        'mapping': not_analyzed
    },
    'related_to': {  # this field keeps a reference to the related item from which metadata has been copied
        'type': 'string',
        'mapping': not_analyzed
    },

    # Audit Information
    'original_creator': Resource.rel('users'),
    'version_creator': Resource.rel('users'),
    'firstcreated': {
        'type': 'datetime'
    },
    'versioncreated': {
        'type': 'datetime'
    },
    'firstpublished': {
        'type': 'datetime',
        'required': False,
        'nullable': True,
    },

    # Ingest Details
    'ingest_provider': Resource.rel('ingest_providers'),
    'source': {     # The value is copied from the ingest_providers vocabulary
        'type': 'string',
        'mapping': not_analyzed
    },
    'original_source': {    # This value is extracted from the ingest
        'type': 'string',
        'mapping': not_analyzed
    },
    'ingest_provider_sequence': {
        'type': 'string',
        'mapping': not_analyzed
    },
    # Copyright Information
    'usageterms': {
        'type': 'string',
        'nullable': True,
    },
    'copyrightnotice': {
        'type': 'string',
        'nullable': True,
        'mapping': not_indexed
    },
    'copyrightholder': {
        'type': 'string',
        'nullable': True
    },
    # Category Details
    'anpa_category': {
        'type': 'list',
        'nullable': True,
        'mapping': {
            'type': 'object',
            'properties': {
                'qcode': not_analyzed,
                'name': not_analyzed,
            }
        }
    },

    'subject': {
        'type': 'list',
        'mapping': {
            'type': 'object',
            'properties': {
                'qcode': not_analyzed,
                'name': not_analyzed
            }
        }
    },
    'genre': {
        'type': 'list',
        'nullable': True,
        'mapping': {
            'type': 'object',
            'properties': {
                'name': not_analyzed,
                'qcode': not_analyzed
            }
        }
    },
    'company_codes': {
        'type': 'list',
        'mapping': {
            'type': 'object',
            'properties': {
                'qcode': not_analyzed,
                'name': not_analyzed,
                'security_exchange': not_analyzed
            }
        }
    },

    # Item Metadata
    ITEM_TYPE: {
        'type': 'string',
        'allowed': content_type,
        'default': 'text',
        'mapping': not_analyzed,
    },
    'package_type': {  # deprecated
        'type': 'string',
        'allowed': ['takes']
    },
    'language': {
        'type': 'string',
        'mapping': not_analyzed,
        'nullable': True,
    },
    'abstract': {
        'type': 'string',
        'nullable': True,
    },
    'headline': {
        'type': 'string'
    },
    'slugline': {
        'type': 'string',
        'mapping': {
            'type': 'string',
            'fielddata': True,
            'fields': {
                'phrase': {
                    'type': 'string',
                    'analyzer': 'phrase_prefix_analyzer',
                    'search_analyzer': 'phrase_prefix_analyzer',
                    'fielddata': True,
                }
            }
        }
    },
    'anpa_take_key': {
        'type': 'string',
        'nullable': True,
    },
    'correction_sequence': {
        'type': 'integer',
        'nullable': True,
        'mapping': not_analyzed
    },
    'rewrite_sequence': {
        'type': 'integer',
        'nullable': True,
        'mapping': not_analyzed
    },
    'rewrite_of': {
        'type': 'string',
        'nullable': True,
        'mapping': not_analyzed,
    },
    'rewritten_by': {
        'type': 'string',
        'nullable': True,
        'mapping': not_analyzed,
    },
    'sequence': {
        'type': 'integer',
        'nullable': True,
    },
    'keywords': {
        'type': 'list',
        'mapping': {
            'type': 'string'
        }
    },
    'word_count': {
        'type': 'integer'
    },
    'priority': {
        'type': 'integer',
        'nullable': True
    },
    'urgency': {
        'type': 'integer',
        'nullable': True
    },
    'profile': {
        'type': 'string',
        'nullable': True,
        'mapping': not_analyzed,
    },

    # Related to state of an article
    ITEM_STATE: {
        'type': 'string',
        'allowed': content_state,
        'mapping': not_analyzed,
    },
    # The previous state the item was in before for example being spiked, when un-spiked it will revert to this state
    'revert_state': {
        'type': 'string',
        'allowed': content_state,
        'mapping': not_analyzed,
    },
    'pubstatus': {
        'type': 'string',
        'allowed': pub_status,
        'default': PUB_STATUS.USABLE,
        'mapping': not_analyzed,
        'nullable': True,
    },
    'signal': {
        'type': 'list',
        'mapping': {
            'type': 'object',
            'properties': {
                'qcode': not_analyzed,
                'name': not_analyzed,
                'scheme': not_analyzed
            }
        }
    },

    BYLINE: {
        'type': 'string',
        'nullable': True,
    },
    'ednote': {
        'type': 'string',
        'nullable': True,
    },
    'authors': {
        'type': 'list',
        'nullable': True,
        'mapping': {
            'type': 'object',
            'dynamic': False,
            'properties': {
                'uri': not_analyzed,
                'parent': not_analyzed,
                'name': not_analyzed,
                'role': not_analyzed,
                'jobtitle': not_enabled,
            }
        }
    },
    'description_text': {
        'type': 'string',
        'nullable': True
    },
    # This is a description of the item as recieved from its source.
    'archive_description': {
        'type': 'string',
        'nullable': True
    },
    'groups': {
        'type': 'list',
        'minlength': 1,
        'nullable': True,
        'mapping': {
            'dynamic': False,
            'properties': {
                'id': not_analyzed,
                'refs': {
                    'dynamic': False,
                    'properties': {
                        'idRef': not_analyzed,
                        '_id': not_analyzed,
                        'uri': not_analyzed,
                        'guid': not_analyzed,
                        'type': not_analyzed,
                        'location': not_analyzed,
                        'headline': {
                            'type': 'string'
                        },
                        'slugline': {
                            'type': 'string'
                        },
                    },
                },
            },
        },
    },
    'deleted_groups': {
        'type': 'list',
        'minlength': 1,
        'nullable': True,
    },
    'body_html': {
        'type': 'string',
        'nullable': True,
        'mapping': {
            'type': 'string',
            'analyzer': 'html_field_analyzer',
            'search_analyzer': 'standard'
        }
    },
    'body_text': {
        'type': 'string',
        'nullable': True,
    },
    'dateline': {
        'type': 'dict',
        'nullable': True,
        'schema': {
            'located': {'type': 'dict', 'nullable': True, 'schema': {
                'state_code': {'type': 'string'},
                'city': {'type': 'string'},
                'tz': {'type': 'string'},
                'country_code': {'type': 'string'},
                'dateline': {'type': 'string'},
                'alt_name': {'type': 'string'},
                'state': {'type': 'string'},
                'city_code': {'type': 'string'},
                'country': {'type': 'string'},
                'code': {'type': 'string'},
                'scheme': {'type': 'string'},
                'location': geopoint,
                'place': {
                    'type': 'dict',
                    'nullable': True,
                    'mapping': not_enabled,
                    'schema': {
                        'code': {'type': 'string'},
                        'name': {'type': 'string'},
                        'qcode': {'type': 'string'},
                        'scheme': {'type': 'string'},
                        'feature_class': {'type': 'string'},
                        'location': geopoint,
                        'continent_code': {'type': 'string', 'nullable': True},
                        'region': {'type': 'string', 'nullable': True},
                        'region_code': {'type': 'string', 'nullable': True},
                        'locality': {'type': 'string', 'nullable': True},
                        'state': {'type': 'string', 'nullable': True},
                        'country': {'type': 'string', 'nullable': True},
                        'world_region': {'type': 'string', 'nullable': True},
                        'locality_code': {'type': 'string', 'nullable': True},
                        'state_code': {'type': 'string', 'nullable': True},
                        'country_code': {'type': 'string', 'nullable': True},
                        'world_region_code': {'type': 'string', 'nullable': True},
                        'rel': {'type': 'string', 'nullable': True},
                        'tz': {'type': 'string', 'nullable': True},
                    },
                },
            }},
            'date': {'type': 'datetime', 'nullable': True},
            'source': {'type': 'string'},
            'text': {'type': 'string', 'nullable': True}
        },
    },
    'expiry': {
        'type': 'datetime'
    },

    # Media Related
    'media': {
        'type': 'file'
    },
    'mimetype': {
        'type': 'string',
        'mapping': not_analyzed
    },
    'poi': {
        'type': 'dict',
        'schema': {
            'x': {'type': 'float', 'nullable': False},
            'y': {'type': 'float', 'nullable': False}
        },
    },
    'renditions': {
        'type': 'dict',
        'schema': {},
        'allow_unknown': True,
    },
    'filemeta': {
        'type': 'dict',
        'schema': {},
        'allow_unknown': True,
    },
    'filemeta_json': {
        'type': 'string'
    },
    'media_file': {
        'type': 'string'
    },
    'contents': {
        'type': 'list'
    },
    ASSOCIATIONS: {
        'type': 'dict',
        'allow_unknown': True,
        'schema': {},
        'mapping': {
            'type': 'object',
            'dynamic': False,
            'properties': {
                'featuremedia': {  # keep indexing featuremedia - we do some filtering using it
                    'type': 'object',
                    'dynamic': False,
                    'properties': {
                        '_id': not_analyzed,
                        'guid': not_analyzed,
                        'unique_id': {'type': 'integer'},
                    }
                }
            }
        }
    },

    # track references to other objects,
    # based on associations but allows queries
    'refs': {
        'type': 'list',
        'readonly': True,
        'schema': {
            '_id': {'type': 'string'},
            'key': {'type': 'string'},
            'uri': {'type': 'string'},
            'guid': {'type': 'string'},
            'type': {'type': 'string'},
        },
        'mapping': {
            'type': 'object',
            'properties': {
                '_id': not_analyzed,
                'key': not_analyzed,
                'uri': not_analyzed,
                'guid': not_analyzed,
                'type': not_analyzed,
            },
        },
    },

    'alt_text': {
        'type': 'string',
        'nullable': True
    },

    # aka Locator as per NewML Specification
    'place': {
        'type': 'list',
        'nullable': True,
        'mapping': {
            'type': 'object',
            'dynamic': False,
            'properties': {
                'scheme': not_analyzed,
                'qcode': not_analyzed,
                'code': not_analyzed,  # content api
                'name': not_analyzed,
                'locality': not_analyzed, # can be used for city/town/village etc.
                'state': not_analyzed,
                'country': not_analyzed,
                'world_region': not_analyzed,
                'locality_code': not_analyzed,
                'state_code': not_analyzed,
                'country_code': not_analyzed,
                'world_region_code': not_analyzed,
                'feature_class': not_analyzed,
                'location': {'type': 'geo_point'},
                'rel': not_analyzed,
            },
        },
    },

    # Not Categorized
    'creditline': {
        'type': 'string'
    },
    LINKED_IN_PACKAGES: {
        'type': 'list',
        'readonly': True,
        'schema': {
            'type': 'dict',
            'schema': {
                PACKAGE: Resource.rel('archive'),
                'package_type': {  # deprecated
                    'type': 'string'
                }
            }
        }
    },
    'highlight': Resource.rel('highlights'),
    'highlights': {
        'type': 'list',
        'schema': Resource.rel('highlights', True)
    },
    'marked_desks': {
        'type': 'list',
        'nullable': True,
        'schema': {
            'type': 'dict',
            'schema': {
                'desk_id': Resource.rel('desks', True),
                'date_marked': {'type': 'datetime', 'nullable': True},
                'user_marked': Resource.rel('users', required=False, nullable=True),
                'date_acknowledged': {'type': 'datetime', 'nullable': True},
                'user_acknowledged': Resource.rel('users', required=False, nullable=True)
            }
        },
    },

    'more_coming': {'type': 'boolean'},  # deprecated

    # Field which contains all the sign-offs done on this article, eg. twd/jwt/ets
    SIGN_OFF: {
        'type': 'string',
        'nullable': True,
    },

    # Desk and Stage Details
    'task': {
        'type': 'dict',
        'schema': {
            'user': {'type': 'string', 'mapping': not_analyzed},
            'desk': {'type': 'string', 'mapping': not_analyzed},
            'stage': {'type': 'string', 'mapping': not_analyzed},
            'status': {'type': 'string', 'mapping': not_analyzed},
        },
    },

    # Task and Lock Details
    'task_id': {
        'type': 'string',
        'mapping': not_analyzed,
        'versioned': False
    },

    'lock_user': Resource.rel('users'),
    'lock_time': {
        'type': 'datetime',
        'versioned': False
    },
    'lock_session': Resource.rel('auth'),

    # Action when the story is locked: edit, correct, kill
    'lock_action': {
        'type': 'string',
        'mapping': not_analyzed,
        'nullable': True
    },

    # template used to create an item
    'template': Resource.rel('content_templates'),

    'body_footer': {  # Public Service Announcements
        'type': 'string',
        'nullable': True,
        'mapping': not_indexed,
    },

    'flags': {
        'type': 'dict',
        'schema': {
            'marked_for_not_publication': {
                'type': 'boolean',
                'default': False
            },
            'marked_for_legal': {
                'type': 'boolean',
                'default': False
            },
            'marked_archived_only': {
                'type': 'boolean',
                'default': False
            },
            'marked_for_sms': {
                'type': 'boolean',
                'default': False
            }
        },
        'default': {
            'marked_for_not_publication': False,
            'marked_for_legal': False,
            'marked_archived_only': False,
            'marked_for_sms': False,
        },
    },

    'sms_message': {
        'type': 'string',
        'mapping': not_analyzed,
        'nullable': True
    },

    FORMAT: {
        'type': 'string',
        'mapping': not_analyzed,
        'default': FORMATS.HTML
    },

    # True indicates that the item has been or is to be published as a result of a routing rule
    'auto_publish': {
        'type': 'boolean'
    },

    # draft-js internal data
    'fields_meta': {
        'type': 'dict',
        'schema': {},
        'allow_unknown': True,
        'nullable': True,
        'mapping': not_enabled,
    },

    'annotations': {
        'type': 'list',
        'mapping': not_enabled,
        'schema': {
            'type': 'dict',
            'schema': {
                'id': {'type': 'integer'},
                'type': {'type': 'string'},
                'body': {'type': 'string'},
            },
        },
    },

    'extra': {
        'type': 'dict',
        'schema': {},
        'mapping': not_enabled,
        'allow_unknown': True,
    },

    'attachments': {
        'type': 'list',
        'nullable': True,
        'schema': {
            'type': 'dict',
            'schema': {
                'attachment': Resource.rel('attachments', nullable=False),
            },
        },
    },

    # references assignment related to the coverage
    'assignment_id': {
        'type': 'string',
        'mapping': not_analyzed
    },

    "translated_from": {
        'type': 'string',
        'mapping': not_analyzed,
    },

    'translation_id': {
        'type': 'string',
        'mapping': not_analyzed,
    },

    'translations': {
        'type': 'list',
        'mapping': not_analyzed,
    },

    # references item id for items auto published using internal destinations
    PROCESSED_FROM: {'type': 'string', 'mapping': not_analyzed},

    # ingested embargoed info, not using embargo to avoid validation
    'embargoed': {'type': 'datetime'},
    'embargoed_text': {'type': 'string', 'mapping': not_indexed},

    'marked_for_user': Resource.rel('users', required=False, nullable=True),

    'broadcast': {
        'type': 'dict',
        'schema': {
            'status': {'type': 'string', 'mapping': not_analyzed},
            'master_id': {'type': 'string', 'mapping': not_analyzed},
            'rewrite_id': {'type': 'string', 'mapping': not_analyzed},
        },
    },

    ITEM_EVENT_ID: {'type': 'string', 'mapping': not_analyzed},

    # schedules
    EMBARGO: {'type': 'datetime', 'nullable': True},
    PUBLISH_SCHEDULE: {'type': 'datetime', 'nullable': True},
    SCHEDULE_SETTINGS: {
        'type': 'dict',
        'schema': {
            'time_zone': {'type': 'string', 'nullable': True, 'mapping': not_analyzed},
            'utc_embargo': {'type': 'datetime', 'nullable': True},
            'utc_publish_schedule': {'type': 'datetime', 'nullable': True},
        },
    },

    # usage tracking
    'used': {'type': 'boolean'},
    'used_count': {'type': 'integer'},
    'used_updated': {'type': 'datetime'},

    # system fields
    '_type': {'type': 'string', 'mapping': None},
    'operation': {'type': 'string'},
    'es_highlight': {'type': 'dict', 'allow_unknown': True, 'readonly': True},

    # targeting fields
    'target_regions': {
        'type': 'list',
        'nullable': True,
        'schema': {
            'type': 'dict',
            'schema': {
                'qcode': {'type': 'string'},
                'name': {'type': 'string'},
                'allow': {'type': 'boolean'}
            }
        }
    },
    'target_types': {
        'type': 'list',
        'nullable': True,
        'schema': {
            'type': 'dict',
            'schema': {
                'qcode': {'type': 'string'},
                'name': {'type': 'string'},
                'allow': {'type': 'boolean'}
            }
        }
    },
    'target_subscribers': {
        'type': 'list',
        'nullable': True
    },
}

metadata_schema['lock_user']['versioned'] = False
metadata_schema['lock_session']['versioned'] = False

crop_schema = {
    'CropLeft': {'type': 'integer'},
    'CropRight': {'type': 'integer'},
    'CropTop': {'type': 'integer'},
    'CropBottom': {'type': 'integer'}
}


def remove_metadata_for_publish(item):
    """Remove metadata from item that should not be public.

    :param item: Item containing the metadata
    :return: item
    """
    from superdesk.attachments import is_attachment_public

    if len(item.get('attachments', [])) > 0:
        item['attachments'] = [attachment for attachment in item['attachments'] if is_attachment_public(attachment)]

    return item


class Priority(SuperdeskBaseEnum):
    """Priority values."""

    Flash = 1
    Urgent = 2
    Three_Paragraph = 3
    Screen_Finance = 4
    Continuous_News = 5
    Ordinary = 6


def get_schema(versioning=False):
    schema = metadata_schema.copy()

    if versioning:
        schema.update({
            '_id_document': {'type': 'string'},
            '_current_version': {'type': 'integer'},
        })

    return schema



from superdesk.tests import TestCase
from apps.rules.routing_rules import RoutingRuleSchemeService


class SetDefaultValuesTestCase(TestCase):
    def test_setting_default_values(self):

        instance = RoutingRuleSchemeService()
        category = [{'qcode': 'a', 'name': 'Australian General News'}]
        result = instance._assign_default_values({'anpa_category': category}, None)
        self.assertEqual(result['anpa_category'], [{'qcode': 'a', 'name': 'Australian General News'}])

        with self.app.app_context():
            result = instance._assign_default_values({}, None)
            self.assertIsNone(result['anpa_category'])

            result = instance._assign_default_values({}, [{'qcode': 'a', 'name': 'Australian General News'}])
            self.assertEqual(result['anpa_category'], [{'qcode': 'a', 'name': 'Australian General News'}])

    def test_getting_selected_categories(self):
        vocabularies = [{'_id': 'categories', 'items': [
            {'qcode': 'a', 'name': 'foo', 'is_active': True},
            {'qcode': 'b', 'name': 'bar', 'is_active': True},
            {'qcode': 'c', 'name': 'baz', 'is_active': False},
        ]}]

        instance = RoutingRuleSchemeService()

        with self.app.app_context():
            self.app.data.insert('vocabularies', vocabularies)
            result = instance._get_categories('a')
            self.assertEqual(result, [{'qcode': 'a', 'name': 'foo'}])

            result = instance._get_categories('a,b')
            self.assertEqual(result, [{'qcode': 'a', 'name': 'foo'}, {'qcode': 'b', 'name': 'bar'}])

            result = instance._get_categories('a,c')
            self.assertEqual(result, [{'qcode': 'a', 'name': 'foo'}])

            result = instance._get_categories('c')
            self.assertEqual(result, [])

            result = instance._get_categories(None)
            self.assertIsNone(result)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import copy
from datetime import timedelta
from unittest.mock import MagicMock, patch

from apps.archive.archive import SOURCE as ARCHIVE
from apps.archive.common import remove_media_files
from superdesk import get_resource_service
from superdesk.media.crop import CropService
from superdesk.tests import TestCase
from superdesk.utc import get_expiry_date, utcnow


NOW = utcnow()


class RemoveSpikedContentTestCase(TestCase):
    articles = [{'guid': 'tag:localhost:2015:69b961ab-2816-4b8a-a584-a7b402fed4f9',
                 '_id': '1',
                 'type': 'text',
                 'last_version': 3,
                 '_current_version': 4,
                 'body_html': 'Test body',
                 'urgency': 4,
                 'headline': 'Two students missing',
                 'pubstatus': 'usable',
                 'firstcreated': utcnow(),
                 'byline': 'By Alan Karben',
                 'ednote': 'Andrew Marwood contributed to this article',
                 'keywords': ['Student', 'Crime', 'Police', 'Missing'],
                 'subject': [{'qcode': '17004000', 'name': 'Statistics'},
                             {'qcode': '04001002', 'name': 'Weather'}],
                 'state': 'draft',
                 'expiry': utcnow() + timedelta(minutes=20),
                 'unique_name': '#1'},
                {'guid': 'tag:localhost:2015:69b961ab-2816-4b8a-a974-xy4532fe33f9',
                 '_id': '2',
                 'last_version': 3,
                 '_current_version': 4,
                 'body_html': 'Test body of the second article',
                 'urgency': 4,
                 'headline': 'Another two students missing',
                 'pubstatus': 'usable',
                 'firstcreated': utcnow(),
                 'byline': 'By Alan Karben',
                 'ednote': 'Andrew Marwood contributed to this article',
                 'keywords': ['Student', 'Crime', 'Police', 'Missing'],
                 'subject': [{'qcode': '17004000', 'name': 'Statistics'},
                             {'qcode': '04001002', 'name': 'Weather'}],
                 'expiry': utcnow() + timedelta(minutes=20),
                 'state': 'draft',
                 'type': 'text',
                 'unique_name': '#2'},
                {'guid': 'tag:localhost:2015:69b961ab-2816-4b8a-a584-a7b402fed4fa',
                 '_id': '3',
                 '_current_version': 4,
                 'body_html': 'Test body',
                 'urgency': 4,
                 'headline': 'Two students missing killed',
                 'pubstatus': 'usable',
                 'firstcreated': utcnow(),
                 'byline': 'By Alan Karben',
                 'ednote': 'Andrew Marwood contributed to this article killed',
                 'keywords': ['Student', 'Crime', 'Police', 'Missing'],
                 'subject': [{'qcode': '17004000', 'name': 'Statistics'},
                             {'qcode': '04001002', 'name': 'Weather'}],
                 'state': 'draft',
                 'expiry': utcnow() + timedelta(minutes=20),
                 'type': 'text',
                 'unique_name': '#3'},
                {'guid': 'tag:localhost:2015:69b961ab-2816-4b8a-a584-a7b402fed4fc',
                 '_id': '4',
                 '_current_version': 3,
                 'state': 'draft',
                 'type': 'composite',
                 'groups': [{'id': 'root', 'refs': [{'idRef': 'main'}], 'role': 'grpRole:NEP'},
                            {
                                'id': 'main',
                                'refs': [
                                    {
                                        'location': 'archive',
                                        'guid': '1',
                                        'residRef': '1',
                                        'type': 'text'
                                    },
                                    {
                                        'location': 'archive',
                                        'residRef': '2',
                                        'guid': '2',
                                        'type': 'text'
                                    }
                                ],
                                'role': 'grpRole:main'}],
                 'firstcreated': utcnow(),
                 'expiry': utcnow() + timedelta(minutes=20),
                 'unique_name': '#4'},
                {'guid': 'tag:localhost:2015:69b961ab-4b8a-a584-2816-a7b402fed4fc',
                 '_id': '5',
                 '_current_version': 3,
                 'state': 'draft',
                 'type': 'composite',
                 'groups': [{'id': 'root', 'refs': [{'idRef': 'main'}, {'idRef': 'story'}], 'role': 'grpRole:NEP'},
                            {
                                'id': 'main',
                                'refs': [
                                    {
                                        'location': 'archive',
                                        'guid': '1',
                                        'residRef': '1',
                                        'type': 'text'
                                    }
                                ],
                                'role': 'grpRole:main'},
                            {
                                'id': 'story',
                                'refs': [
                                    {
                                        'location': 'archive',
                                        'guid': '4',
                                        'residRef': '4',
                                        'type': 'composite'
                                    }
                                ],
                                'role': 'grpRole:story'}],
                 'firstcreated': utcnow(),
                 'expiry': utcnow() + timedelta(minutes=20),
                 'unique_name': '#5'}]

    media = {
        'viewImage': {
            'media': '1592730d582080f4e9fcc2fcf43aa357bda0ed19ffe314ee3248624cd4d4bc54',
            'mimetype': 'image/jpeg',
            'href': 'http://192.168.220.209/api/upload/abc/raw?_schema=http',
            'height': 452,
            'width': 640
        },
        'thumbnail': {
            'media': '52250b4f37da50ee663fdbff057a5f064479f8a8bbd24fb8fdc06135d3f807bb',
            'mimetype': 'image/jpeg',
            'href': 'http://192.168.220.209/api/upload/abc/raw?_schema=http',
            'height': 120,
            'width': 169
        },
        'baseImage': {
            'media': '7a608aa8f51432483918027dd06d0ef385b90702bfeba84ac4aec38ed1660b18',
            'mimetype': 'image/jpeg',
            'href': 'http://192.168.220.209/api/upload/abc/raw?_schema=http',
            'height': 990,
            'width': 1400
        },
        'original': {
            'media': 'stub.jpeg',
            'mimetype': 'image/jpeg',
            'href': 'http://192.168.220.209/api/upload/stub.jpeg/raw?_schema=http',
            'height': 2475,
            'width': 3500
        }
    }

    def test_query_getting_expired_content(self):
        now = utcnow()

        self.app.data.insert(ARCHIVE, [
            {'expiry': get_expiry_date(0), 'state': 'spiked'},
            {'expiry': get_expiry_date(10), 'state': 'spiked'},
            {'expiry': get_expiry_date(20), 'state': 'spiked'},
            {'expiry': get_expiry_date(30), 'state': 'spiked'},
            {'expiry': None, 'state': 'spiked'},
            {'unique_id': 97, 'state': 'spiked'},
            {'expiry': now - timedelta(minutes=10), 'state': 'spiked', 'unique_id': 100},
        ])

        expired_items = get_resource_service(ARCHIVE).get_expired_items(now)
        now = utcnow()
        for expired_items in get_resource_service(ARCHIVE).get_expired_items(now):
            self.assertEquals(1, len(expired_items))
            self.assertEquals(100, expired_items[0]['unique_id'])

    def test_remove_media_files_for_picture(self):
        item = {
            '_id': 'testimage',
            'type': 'picture',
            'renditions': self.media
        }

        original = item.copy()
        with patch.object(self.app.media, 'delete') as media_delete:
            CropService().update_media_references(item, original)
            references_service = get_resource_service('media_references')
            refs = references_service.get(req=None, lookup={'item_id': 'testimage'})
            self.assertEqual(refs.count(), 4)
            for ref in refs:
                self.assertEqual(ref.get('published'), False)
            CropService().update_media_references(item, original, True)
            refs = references_service.get(req=None, lookup={'item_id': 'testimage'})
            for ref in refs:
                self.assertEqual(ref.get('published'), True)

            remove_media_files(item)
            self.assertEqual(0, media_delete.call_count)

            item = {
                '_id': 'testimage2',
                'type': 'picture',
                'renditions': self.media
            }

            original = item.copy()
            CropService().update_media_references(item, original)
            references_service = get_resource_service('media_references')
            refs = references_service.get(req=None, lookup={'item_id': 'testimage2'})
            self.assertEqual(refs.count(), 4)
            for ref in refs:
                self.assertEqual(ref.get('published'), False)

            remove_media_files(item)
            self.assertEqual(0, media_delete.call_count)

            item = {
                '_id': 'testimage3',
                'type': 'picture',
                'renditions': {
                    'viewImage': {
                        'media': '123',
                        'mimetype': 'image/jpeg',
                        'href': 'http://192.168.220.209/api/upload/abc/raw?_schema=http',
                        'height': 452,
                        'width': 640
                    },
                    'thumbnail': {
                        'media': '456',
                        'mimetype': 'image/jpeg',
                        'href': 'http://192.168.220.209/api/upload/abc/raw?_schema=http',
                        'height': 120,
                        'width': 169
                    }
                }
            }

            original = item.copy()
            CropService().update_media_references(item, original)
            references_service = get_resource_service('media_references')
            refs = references_service.get(req=None, lookup={'item_id': 'testimage3'})
            self.assertEqual(refs.count(), 2)
            for ref in refs:
                self.assertEqual(ref.get('published'), False)

            remove_media_files(item)
            self.assertEqual(2, media_delete.call_count)
            for key, rendition in item.get('renditions').items():
                media_delete.assert_any_call(rendition['media'])

    def test_remove_media_files_for_picture_associations(self):
        item = {
            '_id': 'testimage',
            'type': 'text',
            'associations': {
                'featuremedia': {
                    '_id': '123',
                    'renditions': self.media
                },
                'featurevideo': {
                    '_id': '456',
                    'renditions': {
                        'viewImage': {
                            'media': 'testing_123',
                            'mimetype': 'image/jpeg',
                            'href': 'http://192.168.220.209/api/upload/abc/raw?_schema=http',
                            'height': 452,
                            'width': 640
                        },
                        'thumbnail': {
                            'media': 'testing_456',
                            'mimetype': 'image/jpeg',
                            'href': 'http://192.168.220.209/api/upload/abc/raw?_schema=http',
                            'height': 120,
                            'width': 169
                        }
                    }
                }
            }
        }

        original = item.copy()
        with patch.object(self.app.media, 'delete') as media_delete:
            CropService().update_media_references(item, original)
            references_service = get_resource_service('media_references')
            refs = references_service.get(req=None, lookup={'item_id': 'testimage'})
            self.assertEqual(refs.count(), 6)
            for ref in refs:
                self.assertEqual(ref.get('published'), False)
            CropService().update_media_references(item, original, True)
            refs = references_service.get(req=None, lookup={'item_id': 'testimage'})
            for ref in refs:
                self.assertEqual(ref.get('published'), True)

            remove_media_files(item)
            self.assertEqual(0, media_delete.call_count)

    def test_remove_media_files_for_attachments(self):
        attachments = self.app.data.insert('attachments', [{'media': 'foo'}])
        item = {
            '_id': 'test',
            'type': 'text',
            'attachments': [
                {'attachment': attachments[0]},
            ]
        }
        with patch.object(self.app.media, 'delete') as media_delete:
            remove_media_files(item)
        media_delete.assert_any_call('foo', 'attachments')

    def test_delete_by_ids(self):
        ids = self.app.data.insert(ARCHIVE, self.articles)
        archive_service = get_resource_service(ARCHIVE)
        archive_service.on_delete = MagicMock()
        archive_service.delete_by_article_ids(ids)
        self.assertTrue(self.app.data.mongo.is_empty(ARCHIVE))
        self.assertTrue(self.app.data.elastic.is_empty(ARCHIVE))
        self.assertEqual(len(self.articles), archive_service.on_delete.call_count)

    def test_remove_renditions_from_all_versions(self):
        renditions = copy.copy(self.media)

        ids = self.app.data.insert(ARCHIVE, [{
            'state': 'spiked',
            'expiry': get_expiry_date(-10),
            'type': 'picture',
            'renditions': {},
        }])

        self.app.data.insert('archive_versions', [{
            '_id_document': ids[0],
            'type': 'picture',
            'renditions': renditions,
        }])

        with patch.object(self.app.media, 'delete') as media_delete:
            get_resource_service('archive').delete_by_article_ids(ids)
            for key, rendition in renditions.items():
                media_delete.assert_any_call(rendition['media'])

    def _get_original(self, _id):
        return self.app.data.find_one(ARCHIVE, None, _id=_id)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.tests import TestCase

from apps.packages.package_service import PackageService


class PackageServiceTestCase(TestCase):

    def setUp(self):
        self.package1 = {"groups": [{"id": "root",
                                     "refs": [
                                         {
                                             "idRef": "main"
                                         },
                                         {
                                             "idRef": "sidebars"
                                         }
                                     ],
                                     "role": "grpRole:NEP"
                                     },
                                    {
                                    "id": "main",
                                    "refs": [
                                        {
                                            "renditions": {},
                                            "slugline": "Take-1 slugline",
                                            "guid": "123",
                                            "headline": "Take-1 soccer headline",
                                            "location": "archive",
                                            "type": "text",
                                            "itemClass": "icls:text",
                                            "residRef": "123"
                                        },
                                        {
                                            "renditions": {},
                                            "slugline": "Take-3 slugline",
                                            "guid": "789",
                                            "headline": "Take-3 soccer headline",
                                            "location": "archive",
                                            "type": "text",
                                            "itemClass": "icls:text",
                                            "residRef": "789"
                                        }
                                    ],
                                    "role": "grpRole:main"
                                    },
                                    {
                                    "id": "sidebars",
                                    "refs": [
                                        {
                                            "renditions": {},
                                            "slugline": "Take-2 slugline",
                                            "guid": "456",
                                            "headline": "Take-2 soccer headline",
                                            "location": "archive",
                                            "type": "text",
                                            "itemClass": "icls:text",
                                            "residRef": "456"
                                        }
                                    ],
                                    "role": "grpRole:sidebars"
                                    }]}

    def test_remove_ref_from_package(self):
        with self.app.app_context():
            anything_left = PackageService().remove_ref_from_inmem_package(self.package1, "456")
            self.assertEqual(len(self.package1.get('groups', [])), 2)
            root_group = self.package1.get('groups', [])[0]
            self.assertEqual(len(root_group.get('refs', [])), 1)
            self.assertTrue(anything_left)

    def test_remove_two_refs_from_package(self):
        anything_left1 = PackageService().remove_ref_from_inmem_package(self.package1, "456")
        anything_left2 = PackageService().remove_ref_from_inmem_package(self.package1, "123")
        self.assertEqual(len(self.package1.get('groups', [])), 2)
        root_group = self.package1.get('groups', [])[0]
        self.assertEqual(len(root_group.get('refs', [])), 1)
        self.assertTrue(anything_left1)
        self.assertTrue(anything_left2)

    def test_remove_two_refs_from_package2(self):
        PackageService().remove_ref_from_inmem_package(self.package1, "789")
        PackageService().remove_ref_from_inmem_package(self.package1, "123")
        self.assertEqual(len(self.package1.get('groups', [])), 2)
        root_group = self.package1.get('groups', [])[0]
        self.assertEqual(len(root_group.get('refs', [])), 1)

    def test_remove_all_refs_from_package(self):
        anything_left1 = PackageService().remove_ref_from_inmem_package(self.package1, "456")
        anything_left2 = PackageService().remove_ref_from_inmem_package(self.package1, "789")
        anything_left3 = PackageService().remove_ref_from_inmem_package(self.package1, "123")
        self.assertEqual(len(self.package1.get('groups', [])), 1)
        root_group = self.package1.get('groups', [])[0]
        self.assertEqual(len(root_group.get('refs', [])), 0)
        self.assertTrue(anything_left1)
        self.assertTrue(anything_left2)
        self.assertFalse(anything_left3)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import datetime
import os
import unittest
from pytz import utc

from superdesk.etree import etree
from superdesk.io.feed_parsers.afp_newsml_1_2 import AFPNewsMLOneFeedParser


class TestCase(unittest.TestCase):
    def setUp(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.join(dirname, '../fixtures', 'afp.xml')
        provider = {'name': 'Test'}
        with open(fixture, 'rb') as f:
            self.item = AFPNewsMLOneFeedParser().parse(etree.fromstring(f.read()), provider)

    def test_headline(self):
        self.assertEquals(self.item.get('headline'), 'Sweden court accepts receivership for Saab carmaker')

    def test_dateline(self):
        self.assertEquals(self.item.get('dateline', {}).get('text'), 'STOCKHOLM, Aug 29, 2014 (AFP) -')

    def test_slugline(self):
        self.assertEquals(self.item.get('slugline'), 'Sweden-SAAB')

    def test_byline(self):
        self.assertEquals(self.item.get('byline'), '')

    def test_language(self):
        self.assertEquals(self.item.get('language'), 'en')

    def test_guid(self):
        self.assertEquals(self.item.get('guid'), 'urn:newsml:afp.com:20140829T135002Z:TX-PAR-FXW86:1')

    def test_coreitemvalues(self):
        self.assertEquals(self.item.get('type'), 'text')
        self.assertEquals(self.item.get('urgency'), 4)
        self.assertEquals(self.item.get('version'), '1')
        self.assertEquals(self.item.get('versioncreated'), utc.localize(datetime.datetime(2014, 8, 29, 13, 49, 51)))
        self.assertEquals(self.item.get('firstcreated'), utc.localize(datetime.datetime(2014, 8, 29, 13, 49, 51)))
        self.assertEquals(self.item.get('pubstatus'), 'usable')

    def test_subjects(self):
        self.assertEquals(len(self.item.get('subject')), 5)
        self.assertIn({'name': 'automotive equipment', 'qcode': '04011002'}, self.item.get('subject'))
        self.assertIn({'name': 'bankruptcy', 'qcode': '04016007'}, self.item.get('subject'))
        self.assertIn({'name': 'economy, business and finance', 'qcode': '04000000'}, self.item.get('subject'))
        self.assertIn({'name': 'quarterly or semiannual financial statement', 'qcode': '04016038'},
                      self.item.get('subject'))
        self.assertIn({'name': 'manufacturing and engineering', 'qcode': '04011000'}, self.item.get('subject'))

    def test_usageterms(self):
        self.assertEquals(self.item.get('usageterms'), 'NO ARCHIVAL USE')

    def test_genre(self):
        self.assertIn({'name': 'business'}, self.item.get('genre'))
        self.assertIn({'name': 'bankruptcy'}, self.item.get('genre'))

    def test_content_is_text(self):
        self.assertIsInstance(self.item.get('body_html'), type(''))
        self.assertNotRegex(self.item.get('body_html'), '<body.content>')




from behave import then
from steps import json


@then('we get users')
def then_we_get_users(context):
    usernames = json.loads(context.text)
    response_users = json.loads(context.response.get_data())['_items']
    for i in range(len(usernames)):
        username = usernames[i]
        response_username = response_users[i]['username']
        assert username == response_username, \
            'user #{} should be {}, but it was {}'.format(i, username, response_username)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from copy import copy
from eve.utils import config, ParsedRequest

from superdesk.utc import utcnow
from superdesk.services import BaseService
from superdesk.publish.formatters.ninjs_newsroom_formatter import NewsroomNinjsFormatter
from superdesk import get_resource_service
from superdesk.metadata.item import ASSOCIATIONS, remove_metadata_for_publish
from apps.publish.enqueue.enqueue_service import EnqueueService

logger = logging.getLogger('superdesk')


class PublishService(BaseService):
    """A service for publishing to the content api.

    Serves mainly as a proxy to the data layer.
    """

    formatter = NewsroomNinjsFormatter()
    subscriber = {'config': {}}

    def publish(self, item, subscribers=None):
        """Publish an item to content api.

        This must be enabled via ``PUBLISH_TO_CONTENT_API`` setting.

        :param item: item to publish
        """

        if subscribers is None:
            subscribers = []

        if not self._filter_item(item):
            item = EnqueueService.filter_document(item)
            item = remove_metadata_for_publish(item)
            doc = self.formatter._transform_to_ninjs(item, self.subscriber)
            now = utcnow()
            doc.setdefault('firstcreated', now)
            doc.setdefault('versioncreated', now)
            doc.setdefault(config.VERSION, item.get(config.VERSION, 1))
            for _, assoc in doc.get(ASSOCIATIONS, {}).items():
                if assoc:
                    assoc.setdefault('subscribers', [str(subscriber[config.ID_FIELD]) for subscriber in subscribers])
            doc['subscribers'] = [str(sub['_id']) for sub in subscribers]
            if 'evolvedfrom' in doc:
                parent_item = self.find_one(req=None, _id=doc['evolvedfrom'])
                if parent_item:
                    doc['ancestors'] = copy(parent_item.get('ancestors', []))
                    doc['ancestors'].append(doc['evolvedfrom'])
                    doc['bookmarks'] = parent_item.get('bookmarks', [])
                else:
                    logger.warning("Failed to find evolvedfrom item '{}' for '{}'".format(
                        doc['evolvedfrom'], doc['guid'])
                    )

            self._assign_associations(item, doc)
            logger.info('publishing %s to %s' % (doc['guid'], subscribers))
            _id = self._create_doc(doc)
            if 'evolvedfrom' in doc and parent_item:
                self.system_update(parent_item['_id'], {'nextversion': _id}, parent_item)
            return _id
        else:
            return None

    def create(self, docs, **kwargs):
        ids = []
        for doc in docs:
            ids.append(self._create_doc(doc, **kwargs))
        return ids

    def _create_doc(self, doc, **kwargs):
        """Create a new item or update existing."""
        item = copy(doc)
        item.setdefault('_id', item.get('guid'))
        _id = item[config.ID_FIELD] = item.pop('guid')

        # merging the existing and new subscribers
        original = self.find_one(req=None, _id=_id)
        if original:
            item['subscribers'] = list(set(original.get('subscribers', [])) | set(item.get('subscribers', [])))

        self._process_associations(item, original)
        self._create_version_doc(item)
        if original:
            self.update(_id, item, original)
            return _id
        else:
            return super().create([item], **kwargs)[0]

    def _create_version_doc(self, item):
        """
        Store the item in the item version collection
        :param item:
        :return:
        """
        version_item = copy(item)
        version_item['_id_document'] = version_item.pop('_id')
        get_resource_service('items_versions').create([version_item])
        # if the update is a cancel we need to cancel all versions
        if item.get('pubstatus', '') == 'canceled':
            self._cancel_versions(item.get('_id'))

    def _cancel_versions(self, doc_id):
        """
        Given an id of a document set the pubstatus to canceled for all versions
        :param doc_id:
        :return:
        """
        query = {'_id_document': doc_id}
        update = {'pubstatus': 'canceled'}
        for item in get_resource_service('items_versions').get_from_mongo(req=None, lookup=query):
            if item.get('pubstatus') != 'canceled':
                get_resource_service('items_versions').update(item['_id'], update, item)

    def _filter_item(self, item):
        """
        Filter the item out if it matches any API Block filter conditions
        :param item:
        :return: True of the item is blocked, False if it is OK to publish it on the API.
        """
        # Get the API blocking Filters
        req = ParsedRequest()
        filter_conditions = list(get_resource_service('content_filters').get(req=req, lookup={'api_block': True}))

        # No API blocking filters
        if not filter_conditions:
            return False

        filter_service = get_resource_service('content_filters')
        for fc in filter_conditions:
            if filter_service.does_match(fc, item):
                logger.info('API Filter block {} matched for item {}.'.format(fc, item.get(config.ID_FIELD)))
                return True

        return False

    def _assign_associations(self, item, doc):
        """Assign Associations to published item

        :param dict item: item being published
        :param dit doc: ninjs documents
        """
        for assoc, assoc_item in (item.get('associations') or {}).items():
            if not assoc_item:
                continue
            doc.get('associations', {}).get(assoc)['subscribers'] = list(map(str, assoc_item.get('subscribers') or []))

    def _process_associations(self, updates, original):
        """Update associations using existing published item and ensure that associated item subscribers
        are equal or subset of the parent subscribers.
        :param updates:
        :param original:
        :return:
        """
        subscribers = updates.get('subscribers') or []
        for assoc, update_assoc in (updates.get('associations') or {}).items():
            if not update_assoc:
                continue

            if original:
                original_assoc = (original.get('associations') or {}).get(assoc)

                if original_assoc and original_assoc.get(config.ID_FIELD) == update_assoc.get(config.ID_FIELD):
                    update_assoc['subscribers'] = list(set(original_assoc.get('subscribers') or []) |
                                                       set(update_assoc.get('subscribers') or []))

            update_assoc['subscribers'] = list(set(update_assoc['subscribers']) & set(subscribers))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.tests import TestCase


class ExpiredArchiveContentTestCase(TestCase):

    def setUp(self):
        try:
            from apps.archive.commands import RemoveExpiredContent
        except ImportError:
            self.fail("Could not import class under test (RemoveExpiredContent).")
        else:
            self.class_under_test = RemoveExpiredContent
            self.published_items = [
                {
                    '_id': 'item1', 'item_id': 'item1', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', '_current_version': 3,
                    'moved_to_legal': True
                },
                {
                    '_id': 'item2', 'item_id': 'item2', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', '_current_version': 3,
                    'moved_to_legal': True
                },
                {
                    '_id': 'item3', 'item_id': 'item3', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', '_current_version': 3,
                    'moved_to_legal': False
                },
                {
                    '_id': 'item4', 'item_id': 'item4', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', '_current_version': 3,
                    'moved_to_legal': True
                }
            ]

            self.queue_items = [
                {
                    '_id': 'item1', 'item_id': 'item1', 'headline': 'headline',
                    'item_version': 3, 'moved_to_legal': True
                },
                {
                    '_id': 'item2', 'item_id': 'item2', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', 'item_version': 3,
                    'moved_to_legal': True
                },
                {
                    '_id': 'item3', 'item_id': 'item3', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', 'item_version': 3,
                    'moved_to_legal': False
                },
                {
                    '_id': 'item4', 'item_id': 'item4', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', 'item_version': 3,
                    'moved_to_legal': False
                },
                {
                    '_id': 'item5', 'item_id': 'item4', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', 'item_version': 3,
                    'moved_to_legal': True
                }
            ]

            self.app.data.insert('published', self.published_items)
            self.app.data.insert('publish_queue', self.queue_items)

    def test_items_moved_to_legal_success(self):
        test_items = dict()
        test_items['item1'] = self.published_items[0]
        test_items['item2'] = self.published_items[1]
        result = self.class_under_test().check_if_items_imported_to_legal_archive(test_items)
        self.assertDictEqual(result, {})

    def test_items_moved_to_legal_fail_if_published_item_not_moved(self):
        test_items = dict()
        test_items['item2'] = self.published_items[1]
        test_items['item3'] = self.published_items[2]
        result = self.class_under_test().check_if_items_imported_to_legal_archive(test_items)
        self.assertIn('item3', result)

    def test_items_moved_to_legal_fail_if_published_queue_item_not_moved(self):
        test_items = dict()
        test_items['item2'] = self.published_items[1]
        test_items['item3'] = self.published_items[3]
        result = self.class_under_test().check_if_items_imported_to_legal_archive(test_items)
        self.assertIn('item3', result)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from datetime import timedelta
from unittest.mock import patch
from superdesk import get_backend
from superdesk.utc import utcnow
from superdesk.tests import TestCase
from apps.auth.db.reset_password import ResetPasswordService


class UserTokenTestCase(TestCase):

    def setUp(self):
        self.service = ResetPasswordService('tokens', backend=get_backend())

    @patch('apps.auth.db.reset_password.get_random_string', return_value='random')
    def test_store_token(self, get_random_string):
        now = utcnow()
        doc = {'user': 'foo', 'email': 'foo@example.com', '_id': 'foo'}
        with patch.object(self.service.backend, 'create') as create:
            with patch('apps.auth.db.reset_password.utcnow', return_value=now):
                self.service.store_reset_password_token(doc, doc['email'], 10, doc['_id'])
            create.assert_called_with('tokens', [{
                'user': 'foo',
                'email': 'foo@example.com',
                '_id': 'foo',
                '_created': now,
                '_updated': now,
                'expire_time': now + timedelta(days=10),
                'token': 'random',
            }])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
import logging
from superdesk import get_resource_service
from flask import current_app as app
from apps.archive.archive import SOURCE as ARCHIVE
from apps.archive.common import generate_unique_id_and_name, remove_unwanted, insert_into_versions
from superdesk.metadata.item import GUID_TAG, FAMILY_ID, ITEM_STATE, CONTENT_STATE
from superdesk.metadata.utils import generate_guid

logger = logging.getLogger(__name__)


class AppScaffoldDataCommand(superdesk.Command):
    """Initialize the application with some random data fetched from archived.

    Text archive dump must be restored before running this command as well as ``app:initialize_data``
    command to bootstrap the system with desks/users.

    Example:
    ::

        $ python manage.py app:scaffold_data
        $ python manage.py app:scaffold_data --no-of-stories=50

    """

    option_list = [
        superdesk.Option('--no-of-stories', '-n', dest='no_of_stories', default='200')
    ]

    def run(self, no_of_stories):
        logger.info('Starting scaffolding')
        no_of_stories = int(no_of_stories)

        desks = get_resource_service('desks').get(None, {})

        for i, desk in enumerate(desks):
            logger.info('Adding items for desk:' + str(desk['_id']))
            self.ingest_items_for(desk, no_of_stories, i + 1)
        return 0

    def ingest_items_for(self, desk, no_of_stories, skip_index):
        desk_id = desk['_id']
        stage_id = desk['incoming_stage']

        bucket_size = min(100, no_of_stories)

        no_of_buckets = len(range(0, no_of_stories, bucket_size))

        for x in range(0, no_of_buckets):
            skip = x * bucket_size * skip_index
            logger.info('Page : {}, skip: {}'.format(x + 1, skip))
            cursor = get_resource_service('published').get_from_mongo(None, {})
            cursor.skip(skip)
            cursor.limit(bucket_size)
            items = list(cursor)
            logger.info('Inserting {} items'.format(len(items)))
            archive_items = []

            for item in items:
                dest_doc = dict(item)
                new_id = generate_guid(type=GUID_TAG)
                dest_doc[app.config['ID_FIELD']] = new_id
                dest_doc['guid'] = new_id
                generate_unique_id_and_name(dest_doc)

                dest_doc[app.config['VERSION']] = 1
                dest_doc[ITEM_STATE] = CONTENT_STATE.FETCHED
                user_id = desk.get('members', [{'user': None}])[0].get('user')
                dest_doc['original_creator'] = user_id
                dest_doc['version_creator'] = user_id

                from apps.tasks import send_to
                send_to(dest_doc, desk_id=desk_id, stage_id=stage_id, user_id=user_id)
                dest_doc[app.config['VERSION']] = 1  # Above step increments the version and needs to reset
                dest_doc[FAMILY_ID] = item['_id']

                remove_unwanted(dest_doc)
                archive_items.append(dest_doc)

            get_resource_service(ARCHIVE).post(archive_items)
            for item in archive_items:
                insert_into_versions(id_=item[app.config['ID_FIELD']])


superdesk.command('app:scaffold_data', AppScaffoldDataCommand())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""
A module that provides the Superdesk production API application object and runs
the Superdesk production API.

The API is built using the `Eve framework <http://python-eve.org/>`_ and is
thus essentially just a normal `Flask <http://flask.pocoo.org/>`_ application.
"""

import os
import flask
import importlib

from eve import Eve
from eve.io.mongo.mongo import MongoJSONEncoder

from superdesk.datalayer import SuperdeskDataLayer
from superdesk.storage import SuperdeskGridFSMediaStorage
from superdesk.validator import SuperdeskValidator
from superdesk.factory.app import set_error_handlers
from superdesk.factory.sentry import SuperdeskSentry

from prod_api.auth import JWTAuth


def get_app(config=None):
    """
    App factory.

    :param dict config: configuration that can override config
        from `settings.py`
    :return: a new SuperdeskEve app instance
    """

    app_config = flask.Config('.')

    # default config
    app_config.from_object('prod_api.app.settings')

    # https://docs.python-eve.org/en/stable/config.html#domain-configuration
    app_config.update({'DOMAIN': {'upload': {}}})

    # override from instance settings module, but only things defined in default config
    try:
        import settings as server_settings
        for key in dir(server_settings):
            if key.isupper() and key in app_config:
                app_config[key] = getattr(server_settings, key)
    except ImportError:
        pass

    if config:
        app_config.update(config)

    # media storage
    media_storage = SuperdeskGridFSMediaStorage
    if app_config.get('AMAZON_CONTAINER_NAME'):
        from superdesk.storage import AmazonMediaStorage
        media_storage = AmazonMediaStorage

    # auth
    auth = None
    if app_config['PRODAPI_AUTH_ENABLED']:
        auth = JWTAuth

    app = Eve(
        auth=auth,
        settings=app_config,
        data=SuperdeskDataLayer,
        media=media_storage,
        json_encoder=MongoJSONEncoder,
        validator=SuperdeskValidator
    )

    set_error_handlers(app)

    for module_name in app.config.get('PRODAPI_INSTALLED_APPS', []):
        app_module = importlib.import_module(module_name)
        try:
            init_app = app_module.init_app
        except AttributeError:
            pass
        else:
            init_app(app)

    app.sentry = SuperdeskSentry(app)

    return app


if __name__ == '__main__':
    host = '0.0.0.0'
    port = int(os.environ.get('PORT', '5500'))
    app = get_app()
    app.run(host=host, port=port, debug=True, use_reloader=True)



import os
import shutil
from unittest import mock

import superdesk.commands.data_updates
from superdesk import get_resource_service
from superdesk.commands.data_updates import (
    get_data_updates_files, GenerateUpdate, Upgrade, Downgrade
)
from superdesk.tests import TestCase

# change the folder where to store updates for test purpose
DEFAULT_DATA_UPDATE_DIR_NAME = '/tmp/data_updates'
MAIN_DATA_UPDATES_DIR = '/tmp/global_data_updates'


class DataUpdatesTestCase(TestCase):

    def setUp(self):

        dirs = (
            ('DEFAULT_DATA_UPDATE_DIR_NAME', '/tmp/data_updates'),
            ('MAIN_DATA_UPDATES_DIR', '/tmp/global_data_updates'),
        )
        for name, path in dirs:
            shutil.rmtree(path, True)
            os.mkdir(path)
            self.addCleanup(lambda path=path: shutil.rmtree(path))

            patcher = mock.patch('superdesk.commands.data_updates.%s' % name, path)
            self.addCleanup(patcher.stop)
            patcher.start()

        # update the default implementation for `forwards` and `backwards` function
        for n in ('FW', 'BW'):
            name = 'DEFAULT_DATA_UPDATE_%s_IMPLEMENTATION' % n
            patcher = mock.patch('superdesk.commands.data_updates.%s' % name, 'pass')
            self.addCleanup(patcher.stop)
            patcher.start()

    def test_data_update_generation(self):
        assert len(get_data_updates_files()) == 0, get_data_updates_files()
        GenerateUpdate().run(resource_name='RESOURCE_NAME')
        assert len(get_data_updates_files()) == 1, get_data_updates_files()
        GenerateUpdate().run(resource_name='RESOURNCE_NAME')
        assert len(get_data_updates_files()) == 2, get_data_updates_files()

    def test_data_update_generation_create_updates_dir(self):
        updates_dir = DEFAULT_DATA_UPDATE_DIR_NAME
        shutil.rmtree(DEFAULT_DATA_UPDATE_DIR_NAME)
        self.assertFalse(os.path.exists(updates_dir))
        self.app.config['DATA_UPDATES_PATH'] = updates_dir
        GenerateUpdate().run('tmp')
        self.assertTrue(os.path.exists(updates_dir))

    def number_of_data_updates_applied(self):
        return get_resource_service('data_updates').find({}).count()

    def test_dry_data_update(self):
        superdesk.commands.data_updates.DEFAULT_DATA_UPDATE_FW_IMPLEMENTATION = '''
            count = mongodb_collection.find({}).count()
            assert count == 0, count
        '''
        self.assertEqual(self.number_of_data_updates_applied(), 0)
        GenerateUpdate().run(resource_name='data_updates')
        Upgrade().run(dry=True)
        self.assertEqual(self.number_of_data_updates_applied(), 0)

    def test_fake_data_update(self):
        self.assertEqual(self.number_of_data_updates_applied(), 0)
        superdesk.commands.data_updates.DEFAULT_DATA_UPDATE_FW_IMPLEMENTATION = 'raise Exception()'
        superdesk.commands.data_updates.DEFAULT_DATA_UPDATE_BW_IMPLEMENTATION = 'raise Exception()'
        GenerateUpdate().run(resource_name='data_updates')
        self.assertEqual(self.number_of_data_updates_applied(), 0)
        Upgrade().run(fake=True)
        self.assertEqual(self.number_of_data_updates_applied(), 1)
        Downgrade().run(fake=True)
        self.assertEqual(self.number_of_data_updates_applied(), 0)

    def test_data_update(self):
        # create migrations
        for index in range(40):
            superdesk.commands.data_updates.DEFAULT_DATA_UPDATE_FW_IMPLEMENTATION = '''
            assert mongodb_collection
            count = mongodb_collection.find({}).count()
            assert count == %d, count
            assert mongodb_database
            ''' % (index)
            superdesk.commands.data_updates.DEFAULT_DATA_UPDATE_BW_IMPLEMENTATION = '''
            assert mongodb_collection
            count = mongodb_collection.find({}).count()
            assert count == %d, count
            assert mongodb_database
            ''' % (index + 1)
            GenerateUpdate().run(resource_name='data_updates')
        assert self.number_of_data_updates_applied() == 0
        thirdieth_update = get_data_updates_files(True)[29]
        Upgrade().run(data_update_id=thirdieth_update)
        assert self.number_of_data_updates_applied() == 30
        Upgrade().run()
        assert self.number_of_data_updates_applied() == 40
        Downgrade().run()
        assert self.number_of_data_updates_applied() == 39
        Downgrade().run(data_update_id=thirdieth_update)
        assert self.number_of_data_updates_applied() == 29
        Upgrade().run()



import unittest

from apps.preferences import enhance_document_with_default_prefs


class DefaultUserPrefsTestCase(unittest.TestCase):
    def test_enhance(self):
        doc = {'user_preferences': {
            'workqueue:items': ['abc'],
            'editor:theme': {'label': 'Editor'},
        }}
        enhance_document_with_default_prefs(doc)
        self.assertNotIn('label', doc['user_preferences']['editor:theme'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
import logging

from copy import deepcopy

from superdesk import get_resource_service
from eve.utils import ParsedRequest, config
from superdesk.utils import ListCursor
from superdesk.resource import Resource, build_custom_hateoas
from superdesk.services import BaseService
from superdesk.errors import SuperdeskApiError
from superdesk.publish import subscriber_types, SUBSCRIBER_TYPES  # NOQA
from flask import current_app as app
from superdesk.metadata.utils import ProductTypes
from superdesk.notification import push_notification


logger = logging.getLogger(__name__)


class SubscribersResource(Resource):
    schema = {
        'name': {
            'type': 'string',
            'iunique': True,
            'required': True,
            'nullable': False,
            'empty': False
        },
        'media_type': {
            'type': 'string'
        },
        'subscriber_type': {
            'type': 'string',
            'allowed': subscriber_types,
            'required': True
        },
        'sequence_num_settings': {
            'type': 'dict',
            'schema': {
                'min': {'type': 'integer'},
                'max': {'type': 'integer'}
            },
            'required': True
        },
        'email': {
            'type': 'string',
            'empty': False,
            'multiple_emails': True,
            'required': True
        },
        'is_active': {
            'type': 'boolean',
            'default': True
        },
        'is_targetable': {
            'type': 'boolean',
            'default': True
        },
        'critical_errors': {
            'type': 'dict',
            'valueschema': {
                'type': 'boolean'
            }
        },
        'last_closed': {
            'type': 'dict',
            'schema': {
                'closed_at': {'type': 'datetime'},
                'closed_by': Resource.rel('users', nullable=True),
                'message': {'type': 'string'}
            }
        },
        'destinations': {
            'type': 'list',
            'schema': {
                'type': 'dict',
                'schema': {
                    'name': {'type': 'string', 'required': True, 'empty': False},
                    'format': {'type': 'string', 'required': True},
                    'preview_endpoint_url': {'type': 'string'},
                    'delivery_type': {'type': 'string', 'required': True},
                    'config': {'type': 'dict'},
                }
            }
        },
        'products': {
            'type': 'list',
            'schema': Resource.rel('products', True)
        },
        'codes': {
            'type': 'string'
        },
        'global_filters': {
            'type': 'dict',
            'valueschema': {
                'type': 'boolean'
            }
        },
        'content_api_token': {
            'type': 'string',
        },
        'api_products': {
            'type': 'list',
            'schema': Resource.rel('products', True)
        },
        'async': {
            'type': 'boolean',
            'nullable': True,
        },
        'priority': {
            'type': 'boolean',
            'nullable': True,
        },
        'filter_conditions': {
            'type': 'list',
        },
        'content_filters': {
            'type': 'list',
        },
        'selected_subscribers': {
            'type': 'list',
        },
        'init_version': {'type': 'integer'},
    }

    item_methods = ['GET', 'PATCH', 'PUT']

    privileges = {'POST': 'subscribers', 'PATCH': 'subscribers'}

    mongo_indexes = {
        'name_1': ([('name', 1)], {'unique': True}),
    }


class SubscribersService(BaseService):
    def get(self, req, lookup):
        if req is None:
            req = ParsedRequest()
        if req.args and req.args.get('filter_condition'):
            filter_condition = json.loads(req.args.get('filter_condition'))
            return ListCursor(self._get_subscribers_by_filter_condition(filter_condition))
        return super().get(req=req, lookup=lookup)

    def on_create(self, docs):
        for doc in docs:
            self._validate_seq_num_settings(doc)
            self._validate_products_destinations(doc)

    def on_created(self, docs):
        push_notification('subscriber:create', _id=[doc.get(config.ID_FIELD) for doc in docs])

    def on_update(self, updates, original):
        self._validate_seq_num_settings(updates)
        subscriber = deepcopy(original)
        subscriber.update(updates)
        self._validate_products_destinations(subscriber)

    def on_updated(self, updates, original):
        push_notification('subscriber:update', _id=[original.get(config.ID_FIELD)])

    def on_deleted(self, doc):
        get_resource_service('sequences').delete(lookup={
            'key': 'ingest_providers_{_id}'.format(_id=doc[config.ID_FIELD])
        })

    def is_async(self, subscriber_id):
        subscriber = self.find_one(req=None, _id=subscriber_id)
        return subscriber and bool(subscriber.get('async', False))

    def _get_subscribers_by_filter_condition(self, filter_condition):
        """
        Searches all subscribers that has a content filter with the given filter condition

        If filter condition is used in a global filter then it returns all
        subscribers that not disabled the global filter.
        :param filter_condition: Filter condition to test
        :return: List of subscribers
        """
        req = ParsedRequest()
        all_subscribers = list(super().get(req=req, lookup=None))
        selected_products = {}
        selected_subscribers = {}
        selected_content_filters = {}

        filter_condition_service = get_resource_service('filter_conditions')
        content_filter_service = get_resource_service('content_filters')
        product_service = get_resource_service('products')

        existing_products = list(product_service.get(req=req, lookup=None))
        existing_filter_conditions = filter_condition_service.check_similar(filter_condition)
        for fc in existing_filter_conditions:
            existing_content_filters = content_filter_service.get_content_filters_by_filter_condition(fc['_id'])
            for pf in existing_content_filters:
                selected_content_filters[pf['_id']] = pf

                if pf.get('is_global', False):
                    for s in all_subscribers:
                        gfs = s.get('global_filters', {})
                        if gfs.get(str(pf['_id']), True):
                            build_custom_hateoas({'self': {'title': 'subscribers', 'href': '/subscribers/{_id}'}}, s)
                            selected_subscribers[s['_id']] = s

                for product in existing_products:
                    if product.get('content_filter') and \
                        'filter_id' in product['content_filter'] and \
                            product['content_filter']['filter_id'] == pf['_id']:
                        selected_products[product['_id']] = product

                for s in all_subscribers:
                    all_subscriber_products = list(set(s.get('products') or []) | set(s.get('api_products') or []))
                    for p in all_subscriber_products:
                        if p in selected_products:
                            build_custom_hateoas({'self': {'title': 'subscribers', 'href': '/subscribers/{_id}'}}, s)
                            selected_subscribers[s['_id']] = s

        res = {'filter_conditions': existing_filter_conditions,
               'content_filters': list(selected_content_filters.values()),
               'products': list(selected_products.values()),
               'selected_subscribers': list(selected_subscribers.values())}
        return [res]

    def _validate_products_destinations(self, subscriber):
        """Validates the subscribers
            1. At least one destination or one api_products is specified.
            2. If direct products are specified then at least one destination is specified.
        :param dict subscriber:
        :return:
        """
        if not subscriber.get('is_active'):
            return

        if not subscriber.get('destinations') and not subscriber.get('api_products'):
            raise SuperdeskApiError.badRequestError(payload={"destinations": {"required": 1},
                                                             "api_products": {"required": 1}},
                                                    message="At least one destination or one API Product should "
                                                            "be specified")

        if len(subscriber.get("products") or []) and not subscriber.get('destinations'):
            raise SuperdeskApiError.badRequestError(payload={"destinations": {"required": 1}},
                                                    message="Destinations not specified.")

        if subscriber.get('products'):
            lookup = {config.ID_FIELD: {'$in': subscriber.get('products')}, 'product_type': ProductTypes.API.value}
            products = get_resource_service('products').get_product_names(lookup)
            if products:
                raise SuperdeskApiError.badRequestError(payload={'products': 1},
                                                        message="Invalid Product Type. "
                                                                "Products {}.".format(', '.join(products)))
        if subscriber.get('api_products'):
            lookup = {config.ID_FIELD: {'$in': subscriber.get('api_products')},
                      'product_type': ProductTypes.DIRECT.value}
            products = get_resource_service('products').get_product_names(lookup)
            if products:
                raise SuperdeskApiError.badRequestError(payload={'products': 1},
                                                        message="Invalid Product Type. "
                                                                "API Products {}.".format(', '.join(products)))

    def get_subscriber_names(self, lookup):
        """Get the subscriber names based on the lookup.
        :param dict lookup: search criteria
        :return list: list of subscriber name
        """
        subscribers = list(self.get(req=None, lookup=lookup))
        return [subscriber['name'] for subscriber in subscribers]

    def _validate_seq_num_settings(self, subscriber):
        """
        Validates the 'sequence_num_settings' property if present in subscriber.

        Below are the validation rules:
            1.  If min value is present then it should be greater than 0
            2.  If min is present and max value isn't available then it's defaulted to MAX_VALUE_OF_PUBLISH_SEQUENCE

        :return: True if validation succeeds otherwise return False.
        """

        if subscriber.get('sequence_num_settings'):
            min = subscriber.get('sequence_num_settings').get('min', 1)
            max = subscriber.get('sequence_num_settings').get('max', app.config['MAX_VALUE_OF_PUBLISH_SEQUENCE'])

            if min <= 0:
                raise SuperdeskApiError.badRequestError(payload={"sequence_num_settings.min": 1},
                                                        message="Value of Minimum in Sequence Number Settings should "
                                                                "be greater than 0")

            if min >= max:
                raise SuperdeskApiError.badRequestError(payload={"sequence_num_settings.min": 1},
                                                        message="Value of Minimum in Sequence Number Settings should "
                                                                "be less than the value of Maximum")

            del subscriber['sequence_num_settings']
            subscriber['sequence_num_settings'] = {"min": min, "max": max}

        return True

    def generate_sequence_number(self, subscriber):
        """
        Generates Published Sequence Number for the passed subscriber
        """

        assert (subscriber is not None), "Subscriber can't be null"
        min_seq_number = 1
        max_seq_number = app.config['MAX_VALUE_OF_PUBLISH_SEQUENCE']
        if subscriber.get('sequence_num_settings'):
            min_seq_number = subscriber['sequence_num_settings']['min']
            max_seq_number = subscriber['sequence_num_settings']['max']

        return get_resource_service('sequences').get_next_sequence_number(
            key_name='subscribers_{_id})'.format(_id=subscriber[config.ID_FIELD]),
            max_seq_number=max_seq_number,
            min_seq_number=min_seq_number
        )




import pytz
import flask
import babel.dates as dates

from apps.auth import get_user
from eve.render import send_response


bp = flask.Blueprint('locales', __name__)


def get_timezones():
    user = get_user()
    lang = user.get('language', flask.current_app.config.get('DEFAULT_LANGUAGE', 'en')).replace('-', '_')
    return [
        {
            'id': tz,
            'name': dates.get_timezone_name(tz, locale=lang),
            'location': dates.get_timezone_location(tz, locale=lang),
        } for tz in pytz.common_timezones
    ]


@bp.route('/locales/timezones', methods=['GET', 'OPTIONS'])
def locales_view():
    if not flask.current_app.auth.authorized([], 'locales', 'GET'):
        flask.abort(401)
    resp = None
    if flask.request.method == 'GET':
        resp = {'timezones': get_timezones()}
    return send_response(None, (resp, None, None, 200))


def init_app(app):
    bp.url_prefix = '/{}'.format(app.config['URL_PREFIX'].lstrip('/'))
    app.register_blueprint(bp)



import superdesk
import traceback
from superdesk import get_resource_service
from .resource import Resource
from .services import BaseService
import logging


logger = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = 'sequences'
    service = SequencesService(endpoint_name, backend=superdesk.get_backend())
    SequencesResource(endpoint_name, app=app, service=service)


class SequencesResource(Resource):
    schema = {
        'key': {
            'type': 'string',
            'required': True,
            'nullable': False,
            'empty': False,
            'iunique': True
        },
        'sequence_number': {
            'type': 'number',
            'default': 1,
        },
    }
    etag_ignore_fields = ['sequence_number', 'name']
    internal_resource = True
    mongo_indexes = {
        'key_1': ([('key', 1)], {'unique': True}),
    }


class SequencesService(BaseService):

    def get_next_sequence_number(
        self,
        key_name,
        max_seq_number=None,
        min_seq_number=1
    ):
        """
        Generates Sequence Number

        :param key: key to identify the sequence
        :param max_seq_num: default None, maximal possible value, None means no upper limit
        :param min_seq_num: default 1, init value, sequence will start from the NEXT one
        :returns: sequence number
        """
        if not key_name:
            logger.error('Empty sequence key is used: {}'.format('\n'.join(traceback.format_stack())))
            raise KeyError('Sequence key cannot be empty')

        target_resource = get_resource_service('sequences')
        sequence_number = target_resource.find_and_modify(
            query={'key': key_name},
            update={'$inc': {'sequence_number': 1}},
            upsert=True,
            new=True
        ).get('sequence_number')

        if max_seq_number:
            if sequence_number > max_seq_number:
                target_resource.find_and_modify(
                    query={'key': key_name},
                    update={'$set': {'sequence_number': min_seq_number}}
                )

                sequence_number = min_seq_number

        return sequence_number



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from flask import current_app as app, json
from eve.utils import ParsedRequest, config
from eve.methods.common import resolve_document_etag


log = logging.getLogger(__name__)


class BaseService():
    """
    Base service for all endpoints, defines the basic implementation for CRUD datalayer functionality.
    """

    datasource = None

    def __init__(self, datasource=None, backend=None):
        self.backend = backend
        self.datasource = datasource

    def on_create(self, docs):
        pass

    def on_created(self, docs):
        pass

    def on_update(self, updates, original):
        pass

    def on_updated(self, updates, original):
        pass

    def on_replace(self, document, original):
        pass

    def on_replaced(self, document, original):
        pass

    def on_delete(self, doc):
        pass

    def on_deleted(self, doc):
        pass

    def on_fetched(self, doc):
        pass

    def on_fetched_item(self, doc):
        pass

    def create(self, docs, **kwargs):
        ids = self.backend.create(self.datasource, docs, **kwargs)
        return ids

    def update(self, id, updates, original):
        return self.backend.update(self.datasource, id, updates, original)

    def system_update(self, id, updates, original):
        return self.backend.system_update(self.datasource, id, updates, original)

    def replace(self, id, document, original):
        res = self.backend.replace(self.datasource, id, document, original)
        return res

    def delete(self, lookup):
        res = self.backend.delete(self.datasource, lookup)
        return res

    def delete_ids_from_mongo(self, ids):
        res = self.backend.delete_ids_from_mongo(self.datasource, ids)
        return res

    def delete_docs(self, docs):
        return self.backend.delete_docs(self.datasource, docs)

    def find_one(self, req, **lookup):
        res = self.backend.find_one(self.datasource, req=req, **lookup)
        return res

    def find(self, where, **kwargs):
        """Find items in service collection using mongo query.

        :param dict where:
        """
        return self.backend.find(self.datasource, where, **kwargs)

    def get(self, req, lookup):
        if req is None:
            req = ParsedRequest()
        return self.backend.get(self.datasource, req=req, lookup=lookup)

    def get_from_mongo(self, req, lookup, projection=None):
        if req is None:
            req = ParsedRequest()
        if not req.projection and projection:
            req.projection = json.dumps(projection)
        return self.backend.get_from_mongo(self.datasource, req=req, lookup=lookup)

    def find_and_modify(self, **kwargs):
        res = self.backend.find_and_modify(self.datasource, **kwargs)
        return res

    def _validator(self, skip_validation=False):
        resource_def = app.config['DOMAIN'][self.datasource]
        schema = resource_def['schema']
        return (
            None
            if skip_validation
            else app.validator(
                schema, resource=self.datasource, allow_unknown=resource_def['allow_unknown']
            )
        )

    def _resolve_defaults(self, doc):
        validator = self._validator()
        if validator:
            normalized = validator.normalized(doc, always_return_document=True)
            doc.update(normalized)
        return doc

    def post(self, docs, **kwargs):
        for doc in docs:
            self._resolve_defaults(doc)
        self.on_create(docs)
        ids = self.create(docs, **kwargs)
        self.on_created(docs)
        return ids

    def patch(self, id, updates):
        original = self.find_one(req=None, _id=id)
        updated = original.copy()
        self.on_update(updates, original)
        updated.update(updates)
        if config.IF_MATCH:
            resolve_document_etag(updated, self.datasource)
            updates[config.ETAG] = updated[config.ETAG]
        res = self.update(id, updates, original)
        self.on_updated(updates, original)
        return res

    def put(self, id, document):
        self._resolve_defaults(document)
        original = self.find_one(req=None, _id=id)
        self.on_replace(document, original)
        resolve_document_etag(document, self.datasource)
        res = self.replace(id, document, original)
        self.on_replaced(document, original)
        return res

    def delete_action(self, lookup=None):
        if lookup is None:
            lookup = {}
            docs = []
        else:
            docs = list(doc for doc in self.get_from_mongo(None, lookup))
        for doc in docs:
            self.on_delete(doc)
        res = self.delete(lookup)
        for doc in docs:
            self.on_deleted(doc)
        return res

    def is_authorized(self, **kwargs):
        """Subclass should override if the resource handled by the service has intrinsic privileges.

        :param kwargs: should have properties which help in authorizing the request
        :return: ``False`` if unauthorized and True if authorized
        """
        return True

    def search(self, source):
        """Search using search backend.

        :param source: query source param
        """
        return self.backend.search(self.datasource, source)

    def remove_from_search(self, item):
        """Remove item from search.

        :param dict item: item
        """
        return self.backend.remove_from_search(self.datasource, item)


class Service(BaseService):
    pass
